SystemOrganization addCategory: #'L-System'!
SystemOrganization addCategory: #'L-System-Music'!


Object subclass: #LSystemNucleus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

Object subclass: #LSystemParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystemNucleus subclass: #EvolvingNote
	instanceVariableNames: 'note evolver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystemParser subclass: #LSystemTurtle
	instanceVariableNames: 'drawingRules angle unit stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystemTurtle subclass: #HarvestingTurtle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

Object subclass: #AbstractPen
	instanceVariableNames: 'location direction penDown'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

AbstractPen subclass: #CollectingPen
	instanceVariableNames: 'segments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

AbstractPen class
	instanceVariableNames: ''!

AbstractPen subclass: #CanvasPen
	instanceVariableNames: 'canvas width color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

CollectingPen class
	instanceVariableNames: ''!

CanvasPen class
	instanceVariableNames: ''!

LSystemParser class
	instanceVariableNames: ''!

Object subclass: #LSystem
	instanceVariableNames: 'monitor parser axiom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystem class
	instanceVariableNames: ''!

LSystemNucleus class
	instanceVariableNames: ''!

EvolvingNote class
	instanceVariableNames: ''!

LSystemTurtle class
	instanceVariableNames: ''!

HarvestingTurtle class
	instanceVariableNames: ''!

Object subclass: #LSystemMonitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystemMonitor subclass: #NLSystemMonitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

NLSystemMonitor subclass: #MusicalLSystemMonitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System-Music'!

LSystemMonitor subclass: #LSystemRules
	instanceVariableNames: 'rules'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'L-System'!

LSystemMonitor class
	instanceVariableNames: ''!

LSystemRules class
	instanceVariableNames: ''!

NLSystemMonitor class
	instanceVariableNames: ''!

MusicalLSystemMonitor class
	instanceVariableNames: ''!


!LSystem commentStamp: 'spfa 12/13/2019 14:43' prior: 0!
A LSystem is a generic Lindenmayer system.

Any Array of objects can be used as a sequence to be evolved. 
Some objects are reserved for specific semantics though:
	- nil will not be inserted in the sequence
	- an Array is a branching sequence

Rules are handled by the system monitor (a LSystemMonitor), which can also provide state and context to the development process!
]style[(372)!

!LSystemMonitor commentStamp: 'spfa 12/15/2019 11:30' prior: 0!
A LSystemMonitor oversees the development of an Array of objects in a Lindermayer system. It is informed of the start of each expansion step, and asked to finalize the end sequence.

It also drives the expansion itself, either by maintaining (and possibly modifying) production rules, as in subclass LSystemRules, or by using another mechanism (such as expanding nuclei, instances of LSystemNucleus, as in subclass NLSystemMonitor) !
]style[(432)!

!LSystemNucleus commentStamp: 'spfa 12/13/2019 14:52' prior: 0!
A LSystemNucleus knows how to replace itself in a Lindermayer system sequence. When doing so, it is given the L-system monitor which can provide some context (and notably the step count in the sequence expansion)!
]style[(212)!

!LSystemParser commentStamp: 'spfa 12/15/2019 11:31' prior: 0!
A LSystemParser can interpret and process (for example, by displaying it) a sequence generated by a LSystem!
]style[(107)!

!LSystemRules commentStamp: 'spfa 12/15/2019 11:26' prior: 0!
LSystemRules is a LSystem monitor performing expansion via production rules (the usual way to represent Lindemayer system)!
]style[(122)!

!LSystemTurtle commentStamp: 'spfa 12/15/2019 11:28' prior: 0!
A LSystemTurtle interpret a LSystem production (a String) as a list of turtle graphics commands.!
]style[(96)!

!NLSystemMonitor commentStamp: 'spfa 12/13/2019 14:49' prior: 0!
A NLSystemMonitor delegates the expansion of sequence atoms directly to the atoms themselves. All objects are considered as constants, except a LSystemNucleus which is responsible for its own expansion.!
]style[(202)!

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 18:20'!
angle
	
	^ angle ifNil: [90]! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 18:20'!
angle: aNumber
	
	angle := aNumber! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 15:00'!
axiom

	^ axiom! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 15:00'!
axiom: anArray

	axiom := anArray
! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:15'!
brain

	"
	LSystem brain displayOrder: 6

	LSystem brain displayOrder:(1 to: 5)
"

	^ self turtleRules: {
 			$F-> '-F+F+H[+F+H+F]-'.
			$H -> 'HH'} 
		angle: 25
		axiom: 'F'! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:39'!
canvas: aFormCanvas

	canvas := aFormCanvas.
	self reset! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:16'!
cesaroKochFractal

	"LSystem cesaroKochFractal displayOrder:4"

	^ self turtleRules: {$F -> 'F+F--F+F'} angle: 85 axiom: '-F' ! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 23:33'!
cloud

	"
	LSystem cloud displayOrder: 6

	LSystem cloud displayOrder:(1 to: 7)
"

	^ self turtleRules: {
 			$F-> '-F+FG[+F-F]G'} 
		angle: 60
		axiom: 'F'! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:38'!
color

	^ color! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:38'!
color: aColor

	color := aColor! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:44'!
direction
	"Answer the receiver's current direction. 0 is towards the top of the
	screen."

	^direction! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:51'!
direction: anInteger

	direction := anInteger! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 15:09'!
displayOrder: anInteger

	anInteger isInteger ifFalse: [
		^ self parser displaySequences: (anInteger collect: [:n |
			self productionOfOrder: n])].

	self parser displaySequence: (self productionOfOrder: anInteger)

	! !

!LSystemTurtle methodsFor: 'parsing' stamp: 'spfa 12/17/2019 22:45'!
displaySequence: anArray

	self displaySequences: {anArray}! !

!LSystemTurtle methodsFor: 'parsing' stamp: 'spfa 9/13/2020 18:13'!
displaySequences: anArray

	(LSystemPlot new turtle: self copy; sequences: anArray) edit
! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:45'!
down
	"Set the state of the receiver's pen to down (drawing)."

	penDown := true! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:23'!
dragonCurve

	"LSystem dragonCurve displayOrder: 10"

	^ self turtleRules: {
		$X -> 'X+YF+'.
		$Y -> '-FX-Y'.
	} angle: 90 axiom: 'FX'! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/16/2019 11:25'!
drawForward

	^ [:pen | 
		pen down.
		pen go: self unit]! !

!LSystemTurtle methodsFor: 'parsing' stamp: 'spfa 12/17/2019 22:39'!
drawSequence: anArray onCanvas: aCanvas startingAt: aPoint

	| topLeft bottomRight |

	topLeft := bottomRight := aPoint.

	self interpretSequence: anArray 
		with: (CanvasPen new canvas: aCanvas) 
		startingAt: aPoint 
		visitingLocationsWith: [:p |
			topLeft := topLeft min: p.
			bottomRight := bottomRight max: p].

	^ topLeft corner: bottomRight! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/16/2019 11:53'!
drawingRules

	^ drawingRules ifNil: [
		drawingRules := {
			$[ -> #push.
			$] -> #pop.
			$| -> #uTurn.
			$F -> #drawForward.
			$G -> #drawForward.
			$H -> #moveForward.
			$U -> #up.
			$D -> #down.
			$+ -> #right.
			$- -> #left} as: Dictionary]

! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 18:18'!
drawingRules: aDictionary

	drawingRules := aDictionary! !

!EvolvingNote methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:58'!
evolver

	^ evolver! !

!EvolvingNote methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:00'!
evolver: aBlock

	evolver := aBlock! !

!EvolvingNote methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:59'!
expandIn: aLSystemMonitor

	| nn |

	nn := self evolver valueWithEnoughArguments: {note deepCopy. aLSystemMonitor}.
	nn ifNil: [^ {note deepCopy}].

	^ {note deepCopy . self copy note: nn}! !

!LSystemNucleus methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:50'!
expandIn: aLSystemMonitor

	self subclassResponsibility! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:24'!
feather

	"
	LSystem feather displayOrder: 6

	LSystem feather displayOrder:(1 to: 5)
"
	^ self turtleRules: {
 			$F-> '-F+F+H[+F+F]-'.
		"	$H -> 'HH'"} 
		angle: 30
		axiom: 'F'! !

!LSystemMonitor methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:02'!
finalize: anOrderedCollection

	^ anOrderedCollection! !

!MusicalLSystemMonitor methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:05'!
finalize: anOrderedCollection

	^ (anOrderedCollection reject: [:ea | ea isKindOf: LSystemNucleus]) series! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:45'!
go: distance 
	"Move the pen in its current direction a number of bits equal to the 
	argument, distance. If the pen is down, a line will be drawn using the 
	receiver's form source as the shape of the drawing brush."

	self goto: (direction degreeCos @ direction degreeSin) * distance + location! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:18'!
gosperCurve

	"LSystem gosperCurve displayOrder:4"

	^ self turtleRules: {
		$F -> 'F-G--G+F++FF+G-'.
		$G -> '+F-GG--G-F++F+G'.
	} angle: 60 axiom: 'F'
! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:37'!
goto: aPoint 

	| old |
	old := location.
	location := aPoint.
	penDown ifTrue: [self registerSegment: old to: location]
! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:24'!
h1

	"
	LSystem h1 displayOrder: 4

"

	^ self turtleRules: {
 			$F-> 'F[+F]-F'} 
		angle: 30 
		axiom: 'F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:16'!
hilbertCurve

	"
		LSystem hilbertCurve displayOrder: 6
	"

	^ self turtleRules: {
			$A -> '-BF+AFA+FB-'.
			$B -> '+AF-BFB-FA+'.
		} angle: 90 
			axiom: 'A' ! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:25'!
honeyComb
	"
	LSystem honeyComb displayOrder: 5

	LSystem honeyComb displayOrder: (1 to: 5)
"

	^ self turtleRules: {
 			$F-> '-F+F+H[+F+F]-'.
			$H -> 'HH'} 
		angle: 60 
		axiom: 'F'! !

!LSystemMonitor methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:07'!
initializeFirstStep! !

!LSystemMonitor methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:37'!
initializeNextStep! !

!LSystemTurtle methodsFor: 'parsing' stamp: 'spfa 12/17/2019 22:35'!
interpretSequence: anArray with: aPen startingAt: aPoint

	self interpretSequence: anArray with: aPen startingAt: aPoint visitingLocationsWith: [:p | ]
! !

!LSystemTurtle methodsFor: 'parsing' stamp: 'spfa 12/17/2019 22:36'!
interpretSequence: anArray with: aPen startingAt: aPoint visitingLocationsWith: aBlock

	| cp |

	aPen place: (cp := aPoint); north.

	anArray do: [:ea |
		(self drawingRules at: ea ifAbsent: [nil]) ifNotNil: [:sel |
			(self perform: sel) value: aPen.
			aPen location = cp ifFalse: [
				aBlock value: aPen location.
				cp := aPen location]]].
! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/14/2019 18:21'!
iterate: anInteger on: anOrderedCollection

	| col |

	self monitor initializeFirstStep.

	col := anOrderedCollection.
	anInteger timesRepeat: [
		self monitor initializeNextStep.
		col := self iterateOn: col].

	^ self monitor finalize: col! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:28'!
iterateOn: anOrderedCollection

	^ anOrderedCollection species streamContents: [:str |
		anOrderedCollection do: [:ea |
			ea isArray 
				ifTrue: [str nextPut: (self iterateOn: ea)] "branching"
				ifFalse: [str nextPutAll: (self substitute: ea)]]]! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:25'!
k1

	"
	LSystem k1 displayOrder: 3

	LSystem k1 displayOrder:(1 to: 5)
"

	^ self turtleRules: {
 			$F-> 'FF-F+F-F-FF'.} 
		angle: 90 
		axiom: 'F-F-F-F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:26'!
k1b

	"
	LSystem k1b displayOrder:3

	LSystem k1b displayOrder: (1 to: 5)
"

	^ self turtleRules: {
 			$F-> 'FF-F+F-F-FF'.} 
		angle: 60 
		axiom: 'F-F-F-F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:26'!
kochCurve

	"LSystem kochCurve displayOrder: 6"

	^ self turtleRules: {
		$F -> 'F+F--F+F'.
	} angle: 60 axiom: '-F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:26'!
kochIsland

	"LSystem kochIsland displayOrder: (2 to: 4)"

	^ self turtleRules: {
		$F -> 'F+F--F+F'.
	} angle: 60 axiom: '-F--F--F'! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/14/2019 19:15'!
left

	^ [:pen | pen turn: self angle negated]! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/17/2019 22:56'!
levyCurve

	"
	LSystem levyCurve displayOrder: 6

	LSystem levyCurve displayOrder:(4 to: 7)
"

	^ self turtleRules: {$F-> '-F++F-'} 
		angle: 45
		axiom: 'F'! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:44'!
location
	"Answer where the receiver is currently located."

	^location! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:39'!
monitor

	^ monitor! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/14/2019 18:09'!
monitor: aLSystemMonitor

	monitor := aLSystemMonitor! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/16/2019 11:27'!
moveForward

	^ [:pen | 
		pen up.
		pen go: self unit]! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:46'!
north
	"Set the receiver's direction to facing toward the top of the display screen."

	direction := 270! !

!EvolvingNote methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:45'!
note: aMusicalNote

	note := aMusicalNote! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:27'!
nuke

	"
	LSystem nuke displayOrder: 6

	LSystem nuke displayOrder: (1 to: 5)

"

	^ self turtleRules: {
 			$W-> '+++X--F--ZFX+'.
			$X -> '---W++F++YFW-'.
			$Y -> '+ZFX--F--Z+++'.
			$Z -> '-YFW++F++Y---'.} 
		angle: 30  "see also 150"
		axiom: 'W'! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/14/2019 18:09'!
parser

	^ parser! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/14/2019 18:09'!
parser: aLSystemParser

	parser := aLSystemParser! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:27'!
penroseTiling

	"LSystem penroseTiling displayOrder: 4

	LSystem penroseTiling displayOrder: (1 to: 5)
"

	^ self turtleRules: {
			$M -> 'OF++PF----NF[-OF----MF]++'.
			$N -> '+OF--PF[---MF--NF]+'.
			$O -> '-MF++NF[+++OF++PF]-'.
			$P -> '--OF++++MF[+PF++++NF]--NF'.
			$F -> ''} 
		angle: 36 
		axiom: '[N]++[N]++[N]++[N]++[N]'
! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:28'!
pentaplexity

	"
	LSystem pentaplexity displayOrder: 3

	LSystem pentaplexity displayOrder: (1 to: 5)
"


	^ self turtleRules: {
 			$F-> 'F++F++F|F-F++F'} 
		angle: 36 
		axiom: 'F++F++F++F++F'
! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 01:46'!
place: aPoint 
	"Set the receiver at position aPoint. No lines are drawn."

	location := aPoint! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/16/2019 01:50'!
pop

	^ [:pen |
		self stack pop in: [:assoc |
			pen place: assoc key.
			pen direction: assoc value.
			pen down]]! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 15:01'!
productionOfOrder: anInteger

	^ self iterate: anInteger on: self axiom! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/14/2019 19:12'!
push

	^ [:pen |
		self stack push: pen location -> pen direction]! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:36'!
registerSegment: aPoint to: bPoint

	self subclassResponsibility! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:40'!
registerSegment: aPoint to: bPoint

	canvas line: aPoint rounded to: bPoint rounded width: self width color: self color! !

!CollectingPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:46'!
registerSegment: aPoint to: bPoint

	self segments add: (LineSegment from: aPoint to: bPoint)! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:34'!
reset

	location := 0@0.
	direction := 0.
	penDown := true! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:40'!
reset

	super reset.
	width := 1.
	color := Color black! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/14/2019 19:15'!
right

	^ [:pen | pen turn: self angle]! !

!LSystemRules methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:40'!
rules

	^ rules! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 14:36'!
rules: aDictionary

	self monitor: (LSystemRules new rules: aDictionary)! !

!LSystemRules methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:40'!
rules: aDictionary

	rules := aDictionary! !

!CollectingPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:47'!
segments

	^ segments ifNil: [segments := OrderedCollection new]! !

!HarvestingTurtle methodsFor: 'as yet unclassified' stamp: 'spfa 12/17/2019 22:36'!
segmentsInSequence: anArray startingAt: aPoint

	| pen |
	self interpretSequence: anArray with: (pen := CollectingPen new) startingAt: aPoint.
	^ pen segments! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:28'!
sierpinskiArrowHeadCurve

	"LSystem sierpinskiArrowHeadCurve displayOrder: 7

	LSystem sierpinskiArrowHeadCurve displayOrder: (2 to: 7)

"

	^ self turtleRules: {
		$F -> 'G-F-G'.
		$G -> 'F+G+F'.
	} angle: 60 axiom: 'F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:28'!
sierpinskiTriangle

	"LSystem sierpinskiTriangle displayOrder: 6"

	^ self turtleRules: {
		$F -> 'F-G+F+G-F'.
		$G -> 'GG'.
	} angle: 120 axiom: 'F-G-G'! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 19:12'!
stack

	^ stack ifNil: [stack := Stack new]! !

!LSystem methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:36'!
substitute: anObject

	^ self monitor substitute: anObject! !

!LSystemRules methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 16:10'!
substitute: anObject

	^ (self rules at: anObject ifAbsent: [^ {anObject}]) deeplyCollect: #yourself
! !

!NLSystemMonitor methodsFor: 'as yet unclassified' stamp: 'spfa 12/13/2019 13:51'!
substitute: aLSystemNucleus

	(aLSystemNucleus isKindOf: LSystemNucleus) ifFalse: [^ {aLSystemNucleus}].

	^ aLSystemNucleus expandIn: self! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/16/2019 15:28'!
temp

	"
	LSystem temp displayOrder: 3

	LSystem temp displayOrder: (1 to: 5)
"


	^ self turtleRules: {
 			$F-> 'F+FF+F+FF-F-FF'.} 
		angle: 20 
		axiom: 'F+F+F'! !

!LSystem class methodsFor: 'library' stamp: 'spfa 12/18/2019 00:24'!
temp2

	"
	LSystem temp2 displayOrder: 3

	LSystem temp2 displayOrder: (1 to: 5)
"


	^ self turtleRules: {
 			$F-> 'FF'.
			$X -> 'F[+X]F[-X]+X'} 
		angle: 20 
		axiom: 'X'! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:37'!
turn: degrees 

	direction := direction + degrees! !

!LSystem class methodsFor: 'instance creation' stamp: 'spfa 12/15/2019 11:07'!
turtleRules: anArray angle: aNumber 

	| lsys |

	lsys := self new.
	lsys rules: (anArray as: Dictionary).
	lsys parser: LSystemTurtle new.
	lsys parser angle: aNumber.

	^ lsys! !

!LSystem class methodsFor: 'instance creation' stamp: 'spfa 12/16/2019 15:15'!
turtleRules: anArray angle: aNumber axiom: axArray

	^ (self turtleRules: anArray angle: aNumber) axiom: axArray! !

!LSystemTurtle methodsFor: 'commands' stamp: 'spfa 12/16/2019 11:53'!
uTurn

	^ [:pen | pen turn: 180]! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 18:19'!
unit
	
	^ unit ifNil: [10]! !

!LSystemTurtle methodsFor: 'accessing' stamp: 'spfa 12/14/2019 18:40'!
unit: aNumber
	
	unit := aNumber! !

!AbstractPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:37'!
up

	penDown := false! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:38'!
width

	^ width! !

!CanvasPen methodsFor: 'as yet unclassified' stamp: 'spfa 12/16/2019 14:38'!
width: anInteger

	width := anInteger! !
