SystemOrganization addCategory: #ProcessProperties!


Object subclass: #ProcessProperties
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessProperties'!

TestCase subclass: #ProcessPropertiesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessProperties'!

ProcessPropertiesTest class
	instanceVariableNames: ''!

ProcessProperties class
	instanceVariableNames: 'Properties'!


!ProcessProperties commentStamp: '<historical>' prior: 0!
ProcessProperties maintains process-local state in the form of a dictionary associated to each running process.

API:

	ProcessProperties at: #aProp put: 'some object'.
	ProcessProperties at: #aProp.
or
	Processor setProperty: #aProp toValue: 'some object'.
	Processor valueOfProperty: #aProp.

	['some code'] doWithProperty: #aProp -> 'some object'.

	['some code'] doWithProperties: {#aProp -> 'an object'. #bProp -> 'another'} .

Properties are inherited from the parent process (see BlockContext>>#newProcess)!
]style[(513)!

!ProcessorScheduler methodsFor: '*ProcessProperties' stamp: 'spfa 2/25/2010 21:28'!
allProcessesWithProperty: aSymbol do: aBlock

	"aBlock takes two arguments: the process, and its value for property aSymbol"

	ProcessProperties properties finalizeValues.

	^ ProcessProperties properties associationsDo: [:assoc |
			| p |
			p := assoc key.
			(p notNil and: [assoc value includesKey: aSymbol]) 
				ifTrue: [aBlock value: p value: (p valueOfProperty: aSymbol)]]! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
at: aSymbol

	^ (Properties at: Processor activeProcess ifAbsent: [^ nil])
		at: aSymbol ifAbsent: nil! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 2/25/2010 21:18'!
at: aSymbol in: aProcess

	^ (Properties at: aProcess ifAbsent: [^ nil])
		at: aSymbol ifAbsent: nil! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
at: aSymbol put: aValue

	(Properties at: Processor activeProcess ifAbsent: 
		[Properties at: Processor activeProcess put: IdentityDictionary new])
		at: aSymbol put: aValue! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 6/13/2017 13:07'!
doWithProperties: anArray

	| val semaphore exception |

	self flag: #hack. self flag: #toDo.
	"We need an access to an error-raising context.
	At the moment it is not easy to debug something like
		[1/0] doWithProperties: #()
	as can be seen by comparing with
		[1/0] forkWithProperties: #()
	"

	semaphore := Semaphore new.
	[[[val := self value] ensure: [semaphore signal]] on: Exception do: [:ex | exception := ex]] 
		 forkWithProperties: anArray.
	semaphore wait.

	exception ifNotNil: #signal.

	^ val! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:07'!
doWithProperty: anAssociation

	^ self doWithProperties: {anAssociation}
! !

!Symbol methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:16'!
forThisProcess

	^ Processor valueOfProperty: self! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:07'!
forkAt: priority withProperties: anArray

	| p d |

	p := self newProcess.
	p priority: priority.
	d := ProcessProperties properties at: p ifAbsent: [IdentityDictionary new].
	anArray do: [:assoc | d add: assoc].
	ProcessProperties properties at: p put: d.
	^ p resume! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:07'!
forkAt: priority withProperty: anAssociation

	self forkAt: priority withProperties: {anAssociation}
! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:07'!
forkWithProperties: anArray

	| p d |

	p := self newProcess.
	d := ProcessProperties properties at: p ifAbsent: [IdentityDictionary new].
	anArray do: [:assoc | d add: assoc].
	ProcessProperties properties at: p put: d.
	^ p resume! !

!BlockClosure methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:07'!
forkWithProperty: anAssociation

	^ self forkWithProperties: {anAssociation}
! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
initialize

	Properties := WeakIdentityKeyDictionary new.! !

!BlockClosure methodsFor: '*ProcessProperties-override' stamp: 'spfa 11/18/2009 18:07'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."

	| p |

	<primitive: 19> "Simulation guard"
	p := Process	forContext: 
			[self value.
			Processor terminateActive] asContext
		priority: Processor activePriority.
	ProcessProperties properties at: p 
		put: (ProcessProperties properties at: Processor activeProcess
				 ifAbsent: [^ p]) copy.
	^ p! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
properties

	^ Properties! !

!ProcessorScheduler methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:15'!
properties

	^ ProcessProperties properties at: self ifAbsent: [nil]! !

!ProcessProperties class methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
remove: aSymbol

	Properties at: Processor activeProcess ifAbsent: [^ self].
	(Properties at: Processor activeProcess) removeKey: aSymbol
! !

!ProcessorScheduler methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:15'!
setProperty: aSymbol toValue: anObject

	^ ProcessProperties at: aSymbol put: anObject! !

!ProcessPropertiesTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
testBasics

	self assert: (Processor valueOfProperty: #ouhLaLa) isNil.

	self assert: [Processor valueOfProperty: #ouhLaLa] value isNil.

	self assert: ([Processor valueOfProperty: #ouhLaLa] 
						doWithProperty: #ouhLaLa -> #etOui) == #etOui.

	self assert: (Processor valueOfProperty: #ouhLaLa) isNil.
! !

!ProcessPropertiesTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
testInheritance

	| val |

	[[val := Processor valueOfProperty: #ouhLaLa] fork]
						doWithProperty: #ouhLaLa -> #etOui.

	100 milliSeconds asDelay wait.

	self assert: val == #etOui.

	self assert: (Processor valueOfProperty: #ouhLaLa) isNil.
! !

!ProcessPropertiesTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
testInheritance2

	| val level1 level2 |

	[
		[
			val := Processor valueOfProperty: #ouhLaLa.
			level2 := Processor valueOfProperty: #level.
		] 
		doWithProperty: #level -> 2.

		level1 := Processor valueOfProperty: #level
	]
	doWithProperties: {
			#ouhLaLa -> #etOui.
			#level -> 1
		}.

	100 milliSeconds asDelay wait.

	self assert: val == #etOui.
	self assert: level1 == 1.
	self assert: level2 == 2.

	self assert: (Processor valueOfProperty: #ouhLaLa) isNil.
	self assert: (Processor valueOfProperty: #level) isNil.
! !

!ProcessPropertiesTest methodsFor: 'as yet unclassified' stamp: 'spfa 11/18/2009 18:15'!
testSymbolAPI

	self assert: ([#ouhLaLa forThisProcess] 
						doWithProperty: #ouhLaLa -> #etOui) == #etOui.

	self assert: #ouhLaLa forThisProcess isNil.
! !

!Process methodsFor: '*ProcessProperties' stamp: 'spfa 2/25/2010 21:18'!
valueOfProperty: aSymbol

	^ ProcessProperties at: aSymbol in: self! !

!ProcessorScheduler methodsFor: '*ProcessProperties' stamp: 'spfa 11/18/2009 18:15'!
valueOfProperty: aSymbol

	^ ProcessProperties at: aSymbol! !
ProcessProperties initialize!
