SystemOrganization addCategory: #Reactor!


Error subclass: #ReactiveCellParsingError
	instanceVariableNames: 'parsingError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveCellParsingError class
	instanceVariableNames: ''!

TestCase subclass: #ReactorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

Error subclass: #ReactiveCellPropagatingError
	instanceVariableNames: 'initialError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

PluggableMemory subclass: #ReactiveReferencesProvider
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveReferencesProvider class
	instanceVariableNames: ''!

ReactorTest class
	instanceVariableNames: ''!

AlignmentMorph subclass: #ReactiveGrid
	instanceVariableNames: 'table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveCellPropagatingError class
	instanceVariableNames: ''!

Object subclass: #ReactiveCell
	instanceVariableNames: 'formula stringFormula target setter getter formulaValue referenceProvider debugContinuation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveReferencesProvider subclass: #GridReferenceProvider
	instanceVariableNames: 'grid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

GridReferenceProvider class
	instanceVariableNames: ''!

GridReferenceProvider subclass: #ReactiveTable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveTable class
	instanceVariableNames: ''!

Object subclass: #SSSheetBlock
	instanceVariableNames: 'sheet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheetBlock subclass: #SSSheetItself
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheetBlock class
	instanceVariableNames: ''!

SSSheetBlock subclass: #SSSheetMatrix
	instanceVariableNames: 'topLeft bottomRight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheetMatrix class
	instanceVariableNames: ''!

SSSheetItself class
	instanceVariableNames: ''!

LambdaSlotType subclass: #ReactiveReference
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveReference class
	instanceVariableNames: ''!

Error subclass: #ReactorError
	instanceVariableNames: 'updaterError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactorError class
	instanceVariableNames: ''!

ReactiveReferencesProvider subclass: #SSSheet
	instanceVariableNames: 'cells rows columns matrices me'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheet class
	instanceVariableNames: ''!

Object subclass: #Reactor
	instanceVariableNames: 'source updater'
	classVariableNames: 'Parents ProxyDictionary Reactors'
	poolDictionaries: ''
	category: 'Reactor'!

Reactor class
	instanceVariableNames: ''!

SSSheetBlock subclass: #SSSheetColumn
	instanceVariableNames: 'col'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheetColumn class
	instanceVariableNames: ''!

ReactiveGrid class
	instanceVariableNames: ''!

SSSheetBlock subclass: #SSSheetRow
	instanceVariableNames: 'row'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

SSSheetRow class
	instanceVariableNames: ''!

ReactiveCell class
	instanceVariableNames: ''!

BorderedMorph subclass: #ReactiveCellMorph
	instanceVariableNames: 'cell sheet casting slot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Reactor'!

ReactiveCellMorph class
	instanceVariableNames: ''!


!ReactiveCell commentStamp: 'spfa 9/12/2019 01:07' prior: 0!
A ReactiveCell maintains a formula (a LambdaMessageSend) and its associated string form if any, along with a target objet.

Whenever an update is triggered via a Reactor wave of change, the target object is informed of the new value taken by the formula. By default the target is the ReactiveCell itself, which keeps the latest formula value in its inst. var. formulaValue.

The formula must be compiled from its string form by a reference provider (a PluggableMemory or one of its subclasses), responsible for the giving a binding for each unknow symbol in the formula string, associating that symbol to an object wrapped is a specific type of lambda parameter, a ReactiveReference. These objects (appearing as parameters in the formula) are automatically made the 'parents' of the ReactiveCell (in the sense that the cell is made to feature in their list of dependents).


Example with a direct formula specification (not using a string for input):

	| cell morph |

	morph := Morph new.
	cell := ReactiveCell new formula: (morph asReactiveReference color).
	morph color: Color yellow.
	cell formulaValue 			"printIt"

Example with string formula and reference provider. Here the binding is plainly given to the provider, but some providers can define their bindings dynamically (see for example GridReferenceProvider and its subclasses):

	| cell morph pm |

	morph := Morph new.
	pm := PluggableMemory new.
	pm bindings: ({#mimi -> morph asReactiveReference} as: Dictionary).
	cell := ReactiveCell new formulaString: 'mimi color' evaluatedBy: pm.
	morph color: Color yellow.
	cell formulaValue 			"printIt"


Here is a minimal spreadsheet implementation with three cells:

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.
 
	cell1 ownFormulaValue: 0.

	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	cell1 formula: 42.

	cell3 formulaValue		"printIt"


Whenever an Error is raised during the parsing of the formula, or during its evaluation, it is catched by a continuation stored locally, in inst. var. debugContinuation. The target object receives Error as the computation value: It is then its responsibility to handle or discard the continuation. See for example what ReactiveCellMorph does.
!
]style[(2292)!

!ReactiveCellMorph commentStamp: 'spfa 9/15/2019 14:20' prior: 0!
A ReactiveCellMorph is a cell morph in a ReactiveGrid!
]style[(53)!

!ReactiveCellParsingError commentStamp: 'spfa 9/15/2019 14:22' prior: 0!
A ReactiveCellParsingError is raised when the formula string of a ReactiveCell cannot be interpreted!
]style[(100)!

!ReactiveCellPropagatingError commentStamp: 'spfa 9/15/2019 14:25' prior: 0!
A ReactiveCellPropagatingError can be used as the value of a ReactiveCell when the evaluation of one of the cells it depends on has failed.!
]style[(139)!

!ReactiveGrid commentStamp: 'spfa 9/15/2019 14:20' prior: 0!
A ReactiveGrid is the graphical interface of a ReactiveTable!
]style[(60)!

!ReactiveReferencesProvider commentStamp: 'spfa 9/16/2019 15:22' prior: 0!
A ReactiveReferencesProvider is responsible for referencing a set of ReactiveCells, by providing bindings to all symbols meant to name a cell or a group of cells.

It is also informed whenever a cell formula changes.

Basically, it is a spreadsheet. See subclasses (ReactiveReferencesProvider is abstract)!
]style[(305)!

!ReactiveTable commentStamp: 'spfa 8/17/2022 10:48' prior: 0!
A ReactiveTable is a basic spreadsheet with a few cells.

	ReactiveTable new openGrid	"doIt"

Each cell has an editable formula, and an optional casting method complementing the formula and accessible from the cell menu.

An orange cell border signals an input cell: a cell with no dependency and which other cells depend on.

A blue border signals an output cell: a cell with dependency (ultimately to input cells only) which no other cell depend on.

Any output cell can be made the single output of the table. This removes all other output cells, all intermediate cells and leave only the inputs and output.!
]style[(59 26 525),FBitmap DejaVu Sans#14,!

!Reactor commentStamp: 'spfa 9/10/2019 15:35' prior: 0!
Reactor provides a pluggable system for object dependencies, implementing a sort of reactive programming. It uses the generic #dependents system by building over it.

When an object is send #propagateChange (which happens by default in #changed:), the class Reactor checks for updating actions registered for its dependents and run them all. Which may cause some of the dependents to send a #propagateChange either to themselves or to other objects, and so on.

Circular dependencies are prevented at the level of action registration in Reactor (see below). 

Note that #propagateChange does not accept any argument. The nature of the updating operations is entirely defined in the registed actions in Reactor.

Entry points:

#update:after:with:

This is the low-level method for registering an updating action. It will refuses to do so if that leads to circular dependencies.

	| ma mb mc |

	ma := Morph new openInWorld.
	mb := (Morph new color: Color red) topLeft: ma topRight; openInWorld.
	mc := (Morph new color: Color yellow) topLeft: mb topRight; openInWorld.

	Reactor update: mb after: ma with: [:m :n | m color: n color muchLighter].
	Reactor update: mc after: mb with: [:m :n | m color: n color muchLighter].

	"do the above, then change the color of the leftmost morph"


#updatingLambda:

This is an higher-level entry point, where a LambdaMessageSend is given to an arbitrary object, to be evaluated whenever one of the LMS parameter is send #propagateChange. To use any object as a parameter, send it #asReactiveReference.

Note that only one updating lambda form is registered for any given object. #updatingLambda: as a side-effect erase all previously registed updating actions (and if its argument is nil, that's its only effect).

	| ma mb mc |

	ma := Morph new openInWorld.
	mb := (Morph new color: Color red) topLeft: ma topRight; openInWorld.
	mc := (Morph new color: Color yellow) topLeft: mb topRight; openInWorld.

	mb updatingLambda: (Lambda m color: ma asReactiveReference color muchLighter).
	mc updatingLambda: (Lambda m color: mb asReactiveReference color muchLighter).

	"do the above, then change the color of the leftmost morph"
	

The Reactor package depends on packages FunctionalTalk and PluggableMemory

!
]style[(1071 75 1 74 724 3 15 21 19 21 1 3 15 21 19 20 142),f1,,f1,,f1,,f1,,f1,,f1,,f1,,f1,!

!ReactorError commentStamp: 'spfa 9/15/2019 14:21' prior: 0!
A ReactorError is raised whenever an Error is raised during the evaluation of a Reactor updater!
]style[(95)!

!SSSheet commentStamp: 'spfa 9/16/2019 15:48' prior: 0!
A SSSheet (sparse spreadsheet) has virtual columns labelled with any number of letters, and virtual rows labelled with positive integers. It also maintains a collection of rectangular regions, called matrices.

At the moment it is mostly a proof of concept showing how to build a spreadsheet directly out of ReactiveCells. See also ReactiveTable (which has a GUI).

	| sheet |

	sheet := SSSheet new.

	(sheet \#ARY5) formula: 12.
	(sheet \#MU1856) formula: 27.
	(sheet \#Z3) formulaString: 'ARY5 + MU1856'.

	(sheet \#Z3) formulaValue!
]style[(535)!

!SSSheetBlock commentStamp: 'spfa 9/16/2019 15:23' prior: 0!
A SSSheetBlock represents a geometrical part of a SSSheet, and all cells that belong to that part.!
]style[(98)!

!SSSheetColumn commentStamp: 'spfa 9/11/2019 00:09' prior: 0!
A SSSheetColumn represents a column in a SSSheet and all cells in that column!
]style[(77)!

!SSSheetItself commentStamp: 'spfa 9/16/2019 15:46' prior: 0!
A SSSheetItself represents a whole SSSheet from within itself!
]style[(61)!

!SSSheetMatrix commentStamp: 'spfa 9/11/2019 00:09' prior: 0!
A SSSheetMatrix represents a rectangular area in a SSSheet and all cells in there!
]style[(81)!

!SSSheetRow commentStamp: 'spfa 9/11/2019 00:09' prior: 0!
A SSSheetRow represents a row in a SSSheet and all cells in that column!
]style[(71)!

!ReactiveCell methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 16:07'!
!! aReactiveCell

	"Return a LambdaMessageSend which evaluates to a collection of all cells from the receiver to aReactiveCell. How this is defined exactly depends on the current reference provider"

	aReactiveCell referenceProvider == self referenceProvider 
		ifFalse: [self error: 'Unconsistent referencing framework'].

	^ self referenceProvider lambdaCellsFrom: self to: aReactiveCell! !

!ReactiveReferencesProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:16'!
\ aSymbol

	"syntactic sugar"

	^ self cellNamed: aSymbol! !

!GridReferenceProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 21:23'!
acceptsCellMorphMethod: aSymbol

	^ false! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 22:31'!
acceptsCellMorphMethod: aSymbol

	self flag: #toDo.
"
	^ #(rowN colN leftCell) includes: aSymbol
"
	^ false! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:01'!
activeCellsDo: aBlock

	self cellMorphsDo: [:rcm | rcm hasCell ifTrue: [aBlock value: rcm cell]]! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/9/2019 21:50'!
addDeepDependentsTo: anIdentitySet

	self dependents do: [:ea | 
		(anIdentitySet includes: ea) ifFalse: [
			anIdentitySet add: ea.
			ea addDeepDependentsTo: anIdentitySet]].

	^ anIdentitySet! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
all

	^ Reactors ! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
allFor: anObject

	^ (Reactors ifNil: [^ #()]) at: anObject ifAbsent: [^ #()]
	! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:16'!
along: aPoint

	^ self new table: aPoint; yourself! !

!Collection methodsFor: '*Reactor' stamp: 'spfa 9/15/2019 23:46'!
anyOne

	"SPFA note: this is the version from Squeak 5.2, included in Reactor for 5.1"

	"Answer a representative sample of the receiver. This method can
	be helpful when needing to preinfer the nature of the contents of 
	semi-homogeneous collections."

	self do: [:each | ^ each].
	self errorEmptyCollection! !

!Object methodsFor: '*Reactor' stamp: 'spfa 7/24/2022 09:46'!
asReactiveReference

	| lms |

	lms := Lambda o .
	lms slot becomeTransparently: (lms slot as: WeakLambdaSlot).
	lms slot default: self; type: ReactiveReference.
	^ lms
! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/17/2019 17:00'!
belongsToAReferencedRange

	self flag: #toDo.

	^ false "self slot x = 1"! !

!SSSheetMatrix methodsFor: 'accessing' stamp: 'spfa 9/16/2019 09:52'!
bottomRight: aPoint

	bottomRight := aPoint! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:17'!
buildFrom: anArray

	| table |

	table := self along: anArray first.
	table grid populateWith: anArray.

	^ table! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/15/2019 13:27'!
casting

	^ casting ifNil: [#yourself]! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/17/2019 16:49'!
casting: aSelector

	| cf |

	cf := self cell formula ifNil: [^ self].

	cf isLambdaMessageSend 
	ifTrue: [
		cf isUnspecialized 
			ifFalse: [cf := cf etaProtected lambda: aSelector]
			ifTrue: [
				cf selector == self casting
					ifTrue: [cf selector: aSelector]
					ifFalse: [
						aSelector ifNotNil: [
							cf := cf expandedWithMessage: (Message selector: aSelector)]]]]
	ifFalse: [
		aSelector ifNotNil: [cf := cf lambda: aSelector]].

	casting := aSelector.

	self cell formula: cf! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/17/2019 18:13'!
castingIfAny

	^ casting! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/15/2019 13:32'!
castings

	^ {
		'no casting' -> #yourself.
		'cast to number' -> #asNumber.
		'cast to symbol' -> #asSymbol.
		'cast to string' -> #asString.
	} ! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/15/2019 15:01'!
cell

	^ cell ifNil: [cell := ReactiveCell new 
						referenceProvider: self sheet;
						target: self; getter: #contents; setter: #contents:; 
						stringFormula: ReactiveCell notAStringFormula]! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:50'!
cellAt: aPoint

	^ cells at: aPoint ifAbsentPut: [ReactiveCell new referenceProvider: self]
! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 16:15'!
cellAt: aPoint put: anObject

	(self cellAt: aPoint) 
		stringFormula: nil;
		ownFormulaValue: anObject.
	
	^ anObject! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
cellColor

	^ Color grey90! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 22:23'!
cellMorphAt: slot

	self cellMorphsDo: [:rcm | rcm slot = slot ifTrue: [^ rcm]].
	^ nil! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 10:56'!
cellMorphClass

	^ ReactiveCellMorph! !

!GridReferenceProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 21:21'!
cellMorphNamed: aString

	"subclass responsibility"

	^ nil! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 17:35'!
cellMorphNamed: aString

	| col rowi colcode |

	colcode := aString withoutTrailingDigits.
	(colcode allSatisfy: [:ea | ea isLetter and: [ea isUppercase]]) ifFalse: [^ nil].

	col := (Integer readFrom: colcode base: 36) - 9.
	rowi := aString asInteger.

	(rowi isInteger and: [rowi > 0]) ifFalse: [^ nil].

	^ (self grid submorphs at: col + 1) submorphs at: rowi + 1! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/13/2019 09:50'!
cellMorphWithCell: aCell

	self cellMorphsDo: [:rcm | rcm cell == aCell ifTrue: [^ rcm]].
	^ nil! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:00'!
cellMorphsDo: aBlock

	self submorphs allButFirst do: [:cm |
		cm submorphs allButFirst do: [:rcm | aBlock value: rcm]]! !

!ReactiveReferencesProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:13'!
cellNamed: aString

	"subclass responsibility"

	^ nil! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 17:35'!
cellNamed: aString

	| col rowi colcode |

	colcode := aString withoutTrailingDigits.
	(colcode allSatisfy: [:ea | ea isLetter and: [ea isUppercase]]) ifFalse: [^ nil].

	col := (Integer readFrom: colcode base: 36) - 9.
	rowi := aString asInteger.

	(rowi isInteger and: [rowi > 0]) ifFalse: [^ nil].

	^ ((self cellMorphNamed: aString) ifNil: [^ nil]) cell! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:50'!
cellNamed: aString

	^ self cellAt: ((self slotNamed: aString) ifNil: [^ nil])! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 22:42'!
cells

	^ cells! !

!SSSheetBlock methodsFor: 'querying' stamp: 'spfa 9/16/2019 15:29'!
cells.

	"Dynamically compute and provide an unsorted collection of all cells in the block"

	^ self subclassResponsibility! !

!SSSheetColumn methodsFor: 'querying' stamp: 'spfa 9/16/2019 09:48'!
cells.

	^ Array streamContents: [:str |
		sheet cells keysAndValuesDo: [:p :cell |
			(p x = col) ifTrue: [str nextPut: cell]]]

	! !

!SSSheetItself methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:31'!
cells.

	^ sheet cells values! !

!SSSheetMatrix methodsFor: 'querying' stamp: 'spfa 9/16/2019 10:07'!
cells.

	^ Array streamContents: [:str |
		sheet cells keysAndValuesDo: [:p :cell |
			(self includes: p) ifTrue: [str nextPut: cell]]]

	! !

!SSSheetRow methodsFor: 'querying' stamp: 'spfa 9/16/2019 09:48'!
cells.

	^ Array streamContents: [:str |
		sheet cells keysAndValuesDo: [:p :cell |
			(p y = row) ifTrue: [str nextPut: cell]]]

	! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:13'!
cellsLayout

	^ self submorphs size - 1 @ (self submorphs second submorphs size - 1)! !

!Morph methodsFor: '*Reactor-override' stamp: 'spfa 9/19/2019 17:02'!
changed
	"Report that the area occupied by this morph should be redrawn."

	fullBounds 
		ifNil:[self invalidRect: self outerBounds]
		ifNotNil:[self invalidRect: fullBounds].

	Smalltalk at: #ReactorIsReady ifAbsent: [^ self].
	self propagateChange! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/10/2019 14:40'!
changed: aParameter 
	"Receiver changed. The change is denoted by the argument aParameter. 
	Usually the argument is a Symbol that is part of the dependent's change 
	protocol. Inform all of the dependents."

	self dependents do: [:aDependent | aDependent update: aParameter].
	[self propagateChange] on: Error do: []! !

!SSSheetColumn methodsFor: 'accessing' stamp: 'spfa 9/16/2019 10:03'!
col: anInteger sheet: aSSSheet

	col := anInteger.
	sheet := aSSSheet.! !

!ReactiveCellMorph methodsFor: 'querying' stamp: 'spfa 9/12/2019 22:08'!
colN

	^ slot x! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:56'!
colNumber: aString

	| cn r |

	cn := 0.
	r := 1.

	aString reverseDo: [:ea | 
			cn := cn + ((ea asciiValue - 64) * r).
			r := r * 26].

	^ cn! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:23'!
columnAt: col

	| cn |

	cn := col.
	cn isInteger ifFalse: [cn := self colNumber: cn].

	^ self columns at: cn ifAbsentPut: [SSSheetColumn new col: cn sheet: self]! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 22:18'!
columns

	^ columns! !

!ReactiveReference class methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 16:07'!
compose: aLambdaSlot in: aLMS with: bLambdaSlot in: bLMS

	"It is assumed that both LMS are ReactiveCell references

		lms := ReactiveCell new reference   ==>  '/o.{<o> formulaValue}'
	
	The code below does not make much sense otherwise"

	^ (aLambdaSlot lambda: #!! with: bLambdaSlot) collect: #formulaValue! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:11'!
constructor

	^ self constructorWithForginSlot: nil! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:24'!
constructorWithForginSlot: aCellMorph

	^ Array streamContents: [:str | 
		str nextPut: self cellsLayout.
		self cellMorphsDo: [:rcm | 
			str nextPut: (rcm hasCell ifFalse: [nil] 
				ifTrue: [
					rcm == aCellMorph
						ifTrue: [ReactiveTable forgeSlot -> nil]
						ifFalse: [rcm cell formulaString -> rcm castingIfAny]])]]! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
contents

	^ cell ownFormulaValue! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/28/2019 13:48'!
contents: anObject

	((anObject isKindOf: ReactorError) or: [anObject isKindOf: ReactiveCellParsingError]) 
		ifTrue: [
			self cell ownFormulaValue: 
						(ReactiveCellPropagatingError new initialError: anObject)
					keepDebugContinuation: true..
			self stringMorph contents: anObject flagText]
		ifFalse: [
			self cell ownFormulaValue: anObject.
			self updateStringMorph].

	self formulaMorph contents: (cell formulaString ifNil: [''] ifNotNil: [
		(self cell formulaString truncateWithElipsisTo: self maxStringLength)]).

"	self updateFromContents"
	self sheet grid cellMorphsDo: #updateFromContents.

! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/17/2019 18:15'!
contentsForTesting

	self contents in: [:cv |
		(cv isKindOf: Error) ifTrue: [^ cv class name asSymbol].
		cv isLiteral ifTrue: [^ cv].
		^ '## ', (cv printStringLimitedTo: 30)]! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:56'!
copyFromCells

	^ self species buildFrom: self grid constructor! !

!SSSheetMatrix methodsFor: 'testing' stamp: 'spfa 9/16/2019 09:44'!
covers: aPoint to: bPoint

	^ ((aPoint x min: bPoint x) @ (aPoint y min: bPoint y)) = topLeft
		and: [((aPoint x max: bPoint x) @ (aPoint y max: bPoint y)) = bottomRight]! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/11/2019 23:08'!
debugContinuation

	^ debugContinuation
! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/9/2019 21:50'!
deepDependents

	^ self addDeepDependentsTo: IdentitySet new! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 16:57'!
defaultGeometry

	^ 5@5! !

!GridReferenceProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 21:49'!
defineBindingWithKey: aString

	"aString is either a cell coordinate, such as AY17, a,nd the binding will provide the Reactive cell itself, or a cell coordinate followed by a method call, such as AY17color, and then the binding will consider the cell morph as the receiver of the message"

	| meth cm |

	(aString findTokens: '123456789') in: [:ref |
		ref size = 2 ifTrue: [
			meth := ref second]].

	meth ifNil: [
		^ aString 
		-> ((self cellNamed: aString) ifNil: [self error: 'Unknown reference: ', aString]) reference].

	cm := aString allButLast: meth size.
	(self acceptsCellMorphMethod: (meth := meth asSymbol)) 
		ifFalse: [self error: 'Illegal method: ', meth].

	^ aString -> (((self cellMorphNamed: cm) 
		ifNil: [self error: 'Unknown reference: ', aString]) asReactiveReference
			perform: meth asSymbol)
! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:41'!
defineBindingWithKey: aString

	aString = 'sheet' ifTrue: [
		^ aString -> (me := SSSheetItself new sheet: self) asReactiveReference].

	(aString beginsWith: 'row') ifTrue: [
		^ aString -> (self rowAt: (aString allButFirst: 3) asInteger) asReactiveReference].

	(aString beginsWith: 'col') ifTrue: [
		^ aString -> (self columnAt: (aString allButFirst: 3)) asReactiveReference].

	(aString includesSubstring: 'to') ifTrue: [
		^ aString -> (self matrixFrom: (self slotNamed: (aString findTokens: 't') first)
						to: (self slotNamed: (aString findTokens: 'o') second)) 
							asReactiveReference].

	^ aString ->
		((self cellNamed: aString) ifNil: [self error: 'Unknown reference:: ', aString]) reference! !

!ReactiveCellParsingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:42'!
description

	^ 'Parsing error: ', parsingError description! !

!ReactiveCellPropagatingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:50'!
description

	^ 'Propagating error: ', initialError description! !

!ReactorError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:43'!
description

	^ updaterError description! !

!ReactiveCellMorph methodsFor: 'drawing' stamp: 'spfa 9/15/2019 14:32'!
drawOn: aCanvas

	super drawOn: aCanvas.

	(sheet notNil and: [self borderColor ~= sheet cellColor darker]) ifTrue: [
		| d |
		d := 14.
		aCanvas drawPolygon: {self topLeft . self topLeft + (d@0) . self topLeft + (0@d)} fillStyle: self borderColor]! !

!ReactiveCellMorph methodsFor: 'event handling' stamp: 'spfa 9/15/2019 16:52'!
editFormulaEvt: evt

	(self hasCell and: [evt yellowButtonChanged]) ifTrue: [^ self popUpMenuEvt: evt].

	(self hasCell not and: [evt yellowButtonChanged not]) ifTrue: [^ self].

	evt wasHandled: true.
	self queryFormula.

! !

!ReactiveCell methodsFor: 'debugging' stamp: 'spfa 9/12/2019 12:11'!
eraseDebugContinuation

	debugContinuation := nil
! !

!ReactiveCellMorph methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 14:52'!
eraseReactiveTableTemplateForForging

	MuO context compositionEnvironment 
		eraseReactiveTableTemplateForForging: cell ownFormulaValue className! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:37'!
example1

	"	ReactiveTable example1 openGrid	"

	^ self buildFrom:  {5@5 . '1'->nil . 'A1+5'->nil . 'A2*6'->nil . nil . '12'->nil . nil . nil . nil . nil . nil . 'X  sin'->nil . nil . nil . '(X + 1) cos'->nil . 'C4 + A5'->nil . nil . 'C5 o: C1'->nil . 'C1 o: C5'->nil . nil . 'E3 <~ 0.5'->#asString . '42'->nil . nil . 'D3'->nil . nil . 'E3 value: 0.5'->nil} ! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:38'!
example2

	"	ReactiveTable example2 openGrid	"

	^ self buildFrom: {5@5 . '1'->nil . 'A1+5'->nil . 'A2*5'->nil . 'A3 sqrt'->nil . nil . nil . 'B4 '->#asNumber . nil . 'A3 sqrt'->#asString . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}.! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/18/2019 00:42'!
example3

	"	ReactiveTable example3 openGrid	"

	^ self buildFrom: {5@5 . '1'->nil . 'A1+5'->nil . 'A2*5'->nil . 'A3 sqrt'->nil . nil . 'B2'->#asSymbol . 'B4 '->#asNumber . 'B2 +10'->nil . 'A3 sqrt'->#asString . nil . 'B1 reverse'->nil . 'C1 first: 2'->#asNumber . 'C2+B2'->nil . 'C3 rounded'->nil . 'C4'->#asSymbol . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}! !

!ReactiveCellParsingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 11:05'!
flagText

	^ '-Bad Formula-'! !

!ReactorError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 11:04'!
flagText

	^ (updaterError isKindOf: ReactiveCellPropagatingError) 
		ifTrue: ['-No Ref-']
		ifFalse: ['-Error-']
! !

!ReactiveCellMorph methodsFor: 'initialization' stamp: 'spfa 2/17/2020 21:26'!
forRowLabel: anInteger

	self color: Color transparent; borderColor: Color transparent.
	self formulaMorph contents: anInteger asString! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:24'!
forge: anObject

	self grid cellMorphsDo: [:rcm |
		rcm cell formulaString = self class forgeSlot ifTrue: [
			rcm formula: anObject.
			^ self]].

	self grid cellMorphsDo: [:rcm |
		rcm hasCell ifFalse: [
			rcm formula: anObject.
			^ self]].! !

!ReactiveCellMorph methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:22'!
forgeConstructor

	^ sheet grid constructorWithForginSlot: self! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:24'!
forgeSlot

	^ '#''<forge slot>'''! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
formula

	^ formula ! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/18/2019 00:53'!
formula: aLMS

	| ulam |

	formula := aLMS.

	ulam := formula isUnspecialized
		ifTrue: [Lambda m formulaValue: formula]
		ifFalse: [(Lambda m formulaValue: Lambda f) <<= (#f -> formula lambda)].

	self updatingLambda: ulam trigger: true.

	(self referenceProvider ifNil: [^ self]) formulaChangedIn: self
! !

!ReactiveCellMorph methodsFor: 'input' stamp: 'spfa 9/28/2019 13:49'!
formula: anObject

	self cell formula: anObject.
	self cell stringFormula: nil.
	self formulaMorph contents: ''.
	self updateStringMorph; updateFromContents! !

!ReactiveReferencesProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:15'!
formulaChangedIn: aCell

	"Subclasses may do whatever needed when the formula of a cell has just changed. This will happen right after the propagation of change has taken place"! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:08'!
formulaChangedIn: aCell

	| place row col |

	place := cells keyAtValue: aCell. 
	row := place y.
	col := place x.

	(self referencedRowAt: row) propagateChange.
	(self referencedColumnAt: col) propagateChange.
	self matricesIncluding: col@row do: #propagateChange.! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
formulaMorph

	^ self submorphs second ! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
formulaString

	^ stringFormula! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/11/2019 10:21'!
formulaString: aString

	self formulaString: aString evaluatedBy: self referenceProvider! !

!ReactiveCellMorph methodsFor: 'input' stamp: 'spfa 9/18/2019 01:02'!
formulaString: aString

	cell formulaString: aString.
	self formulaMorph contents: aString.
	self updateStringMorph; updateFromContents! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/15/2019 10:16'!
formulaString: aString evaluatedBy: aPluggableMemory

	debugContinuation := nil.

	stringFormula := aString.
	self formula: ([aPluggableMemory evalString: aString] on: Error do: [:ex | 
			debugContinuation := Continuation current.
			debugContinuation isContinuation
				ifTrue: [ReactiveCellParsingError new parsingError: ex]
				ifFalse: [ex signal]])

! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/13/2019 09:06'!
formulaValue

	^ self! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/15/2019 11:03'!
formulaValue

	(self target perform: getter) in: [:fval |
		(fval isKindOf: ReactiveCellPropagatingError) ifTrue: [fval signal].
		^ fval]! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/10/2019 11:57'!
formulaValue: anObject

	self target perform: setter with: anObject! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
free: anObject

	(Parents at: anObject ifAbsentPut: [#()]) do: [:p | p removeDependent: anObject].
	Parents removeKey: anObject.
	Reactors removeKey: anObject ifAbsent: []! !

!SSSheetMatrix class methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:09'!
from: aPoint to: bPoint in: aSSSheet

	^ self new
		topLeft: (aPoint x min: bPoint x) @ (aPoint y min: bPoint y);
		bottomRight: (aPoint x max: bPoint x) @ (aPoint y max: bPoint y);
		sheet: aSSSheet! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
getter

	^ getter! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
getter: aSelector

	getter := aSelector! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 16:58'!
grid

	grid ifNil: [self table: self defaultGeometry].
	^ grid! !

!GridReferenceProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:52'!
grid: aMorph

	grid := aMorph.! !

!ReactiveCellMorph methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:07'!
hardCodeAsForgingTemplate

	cell ownFormulaValue class class compile: 'reactiveTableTemplateForForging

	^ ', self forgeConstructor
	classified: 'interaction'
! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/14/2019 23:27'!
hasCell

	self flag: #toDo. "bad test name"

	^ cell notNil and: [cell formulaString ~= ReactiveCell notAStringFormula]! !

!SSSheetMatrix methodsFor: 'testing' stamp: 'spfa 9/16/2019 09:55'!
includes: aPoint

	^ aPoint x >= topLeft x
		and: [aPoint x <= bottomRight x]
		and: [aPoint y >= topLeft y]
		and: [aPoint y <= bottomRight y]! !

!ReactiveCellParsingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:52'!
initialError

	^ parsingError! !

!ReactiveCellPropagatingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:17'!
initialError

	^ initialError! !

!ReactorError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 11:02'!
initialError

	^ (updaterError isKindOf: ReactiveCellPropagatingError) 
		ifTrue: [updaterError initialError]
		ifFalse: [updaterError]! !

!ReactiveCellPropagatingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 11:01'!
initialError: anError

	initialError := anError initialError! !

!ReactiveCell methodsFor: 'initialize-release' stamp: 'spfa 9/14/2019 23:22'!
initialize

	"by default, the cell itself is its own target and maintains its formula value in inst var formulaValue"

	target := nil. "which means self, see #target"
	getter := #ownFormulaValue.
	setter := #ownFormulaValue:.! !

!ReactiveCellMorph methodsFor: 'initialization' stamp: 'spfa 2/17/2020 23:47'!
initialize

	super initialize.

	self
		layoutPolicy: TableLayout new;
		listDirection: #topToBottom;
		hResizing: #spaceFill;
		cellInset: 5;
		wrapCentering: #center;
		layoutInset: 0;
		vResizing: #spaceFill;
		rubberBandCells: true.

	self addMorph: (StringMorph contents: '').
	self addMorph: (StringMorph contents: '' font: MuOGUI mediumFont).
! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 10:51'!
initialize
	"initialize the state of the receiver"
	super initialize.

	! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/19/2019 17:02'!
initialize

	"Reactor initialize"

	Reactors  := WeakIdentityKeyDictionary new.
	Parents := WeakIdentityKeyDictionary new.
	Smalltalk at: #ReactorIsReady put: true
! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 11:06'!
initialize

	cells := Dictionary new.

	rows := Dictionary new.
	columns := Dictionary new.
	matrices := IdentitySet new.! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/17/2019 16:57'!
isInputCell

	self hasCell ifFalse: [^ false]. 
	(self belongsToAReferencedRange not and: [cell dependents isEmpty]) ifTrue: [^ false].
	cell formula isLambdaMessageSend ifFalse: [^ true].
	cell formula isUnspecialized ifFalse: [^ true].
	^ cell formula boundSlots isEmpty! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/13/2019 23:38'!
isIntermediateCell

	"?? see also in  ReactiveCell>>#pipeThroughParentCells"

	cell ifNil: [^ false].
	cell formula isLambdaMessageSend ifFalse: [^ false].
"
	^ cell formula boundSlots noneSatisfy: [:ea | ea default isKindOf: ReactiveCell].
"
	^ cell dependents notEmpty! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/17/2019 13:45'!
isOutputCell

	self hasCell ifFalse: [^ false]. 
	cell formula isLambdaMessageSend ifFalse: [^ false].
	cell formula isUnspecialized ifFalse: [^ false].
	^ cell formula boundSlots notEmpty! !

!ReactiveCellMorph methodsFor: 'testing' stamp: 'spfa 9/14/2019 23:57'!
isTerminalCell

	^ self isOutputCell and: [self isIntermediateCell not]! !

!ReactiveCellMorph methodsFor: 'composing' stamp: 'spfa 9/17/2019 16:29'!
keepAsSingleOutput

	self pipeToInputCells.
	sheet grid resetAllOutputBut: self.! !

!GridReferenceProvider methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 14:40'!
lambdaCellsFrom: aCell to: bCell

	self subclassResponsibility! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 23:22'!
lambdaCellsFrom: aCell to: bCell

	| acm bcm maxx maxy minx miny|

	acm := self grid cellMorphWithCell: aCell.
	bcm := self grid cellMorphWithCell: bCell.

	maxx := bcm slot x max: acm slot x.
	minx := bcm slot x min: acm slot x.
	maxy := bcm slot y max: acm slot y.
	miny := bcm slot y min: acm slot y.

	^ Array streamContents: [:str |
		self grid cellMorphsDo: [:rcm |
			 rcm hasCell ifTrue: [
				(rcm slot x >= minx
					and: [rcm slot x <= maxx]
					and: [rcm slot y >= miny]
					and: [rcm slot y <= maxy])
						ifTrue: [
							str nextPut: rcm cell]]]]! !

!ReactiveCellMorph methodsFor: 'querying' stamp: 'spfa 9/12/2019 22:28'!
leftCell

	^ (sheet grid cellMorphAt: (slot x - 1) @ slot y) cell formulaValue! !

!SSSheetColumn methodsFor: 'querying' stamp: 'spfa 9/16/2019 10:32'!
loc: anInteger

	^ (sheet cells at: col @ anInteger ifAbsent: [^ nil]) formulaValue
	! !

!SSSheetMatrix methodsFor: 'querying' stamp: 'spfa 9/16/2019 10:31'!
loc: aPoint

	^ (sheet cells at: topLeft + aPoint - (1@1) ifAbsent: [^ nil]) formulaValue! !

!SSSheetRow methodsFor: 'querying' stamp: 'spfa 9/16/2019 10:33'!
loc: anInteger

	^ (sheet cells at: anInteger @ row ifAbsent: [^ nil]) formulaValue
	! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:17'!
matrices

	^ matrices! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 12:12'!
matricesIncluding: aPoint do: aBlock

	matrices do: [:m | (m includes: aPoint) ifTrue: [aBlock value: m]]! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:09'!
matrixFrom: aPoint to: bPoint

	^ matrices detect: [:mx | mx covers: aPoint to: bPoint] 
		ifNone: [matrices add: (SSSheetMatrix from: aPoint to: bPoint in: self)]! !

!ReactiveCellMorph methodsFor: 'updating' stamp: 'spfa 9/9/2019 21:50'!
maxStringLength

	^ 30! !

!ReactiveCell class methodsFor: 'as yet unclassified' stamp: 'spfa 9/14/2019 23:26'!
notAStringFormula

	"return a symbol intended to signal that a ReactiveCell is to be considered as not initialized (in the context of some algorithm) - will be stored in the cell string formula, which it is not!!"

	^ #NoCellHere! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:19'!
openGrid

	^ self grid openInHand! !

!ReactiveTable class methodsFor: 'as yet unclassified' stamp: 'spfa 10/3/2019 14:02'!
openNew

	^ self new openGrid! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/13/2019 23:10'!
overwriteFormulaWith: aLMS

	formula := aLMS.
! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
ownFormulaValue

	^ formulaValue! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/15/2019 10:25'!
ownFormulaValue: anObject

	self ownFormulaValue: anObject keepDebugContinuation: false! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/15/2019 10:25'!
ownFormulaValue: anObject keepDebugContinuation: aBoolean

	aBoolean ifFalse: [debugContinuation := nil].

	anObject == formulaValue ifTrue: [^ self].

	formulaValue := anObject.
	self propagateChange! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
parentsOf: anObject

	^ Parents at: anObject ifAbsent: [^ #()]! !

!ReactiveCellParsingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:13'!
parsingError

	^ parsingError! !

!ReactiveCellParsingError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:12'!
parsingError: anError

	parsingError := anError! !

!ReactiveCell methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 16:00'!
pipeThroughParentCells

	| eslots |

	self formula isLambdaMessageSend ifFalse: [^ #done].
	self formula isUnspecialized ifFalse: [^ #done].

	eslots := self formula boundSlots select: [:ea | 
		(ea default formula isLambdaMessageSend
			and: [ea default formula isUnspecialized]
			and: [ea default formula boundSlots notEmpty])]
		thenCollect: [:ea | ea -> ea default formula].

	eslots ifEmpty: [
		self formula: self formula "simplified" "see why this fails".
		^ #done].

	stringFormula := nil.

	formula := self formula <<==!! eslots! !

!ReactiveCell methodsFor: 'as yet unclassified' stamp: 'spfa 9/18/2019 00:31'!
pipeToInputCells

	| rules lms |

	lms := Lambda lms type: (SlotOfKind class: LambdaMessageSend).
	rules := {lms quoted formulaValue -> lms}.

	[self pipeThroughParentCells == #done] whileFalse.

	formula := formula upwardReduceWithRules: rules! !

!ReactiveCellMorph methodsFor: 'composing' stamp: 'spfa 9/18/2019 00:38'!
pipeToInputCells

	| str cc |

	"we preserve the casting so that it is kept interactive"
	cc := casting.
	cc ifNotNil: [	self casting: nil].

	self cell pipeToInputCells.

	self cell formula isLambdaMessageSend ifFalse: [^ self].
	self cell formula isUnspecialized ifFalse: [^ self].

	str := ((cell formula printString) copyAfter: ${) copyUpTo: $}.

	cc ifNotNil: [	self casting: cc].

	self cell formula boundSlots do: [:ea |
		str := str copyReplaceAll: ea asString
				with: ((sheet bindings associations detect: [:assoc |
							assoc value slots anyOne default == ea default]) key)].

	([self casting value: (self cell referenceProvider evalString: str) defaultForm value] 
		on: Error do: [#doh]) = self contents ifFalse: [
		"The aggregated formula is not reliable - forget it"
		str := '#Unwritten'].

	str := str copyReplaceAll: ' formulaValue' with: ''.
	str := str copyReplaceAll: ' collect: #formulaValue' with: ''.

	self cell stringFormula: str. "note: this does not trigger an evaluation"
	self formulaMorph contents: (str truncateWithElipsisTo: self maxStringLength).

	self cell formula boundSlots anyOne default propagateChange.
! !

!ReactiveCellMorph methodsFor: 'event handling' stamp: 'spfa 9/28/2019 15:15'!
popUpMenuEvt: evt

	| menu |

	evt yellowButtonChanged ifFalse: [^ self].

	menu := MenuMorph new defaultTarget: self.

	cell debugContinuation ifNotNil: [
		menu add: 'DEBUG' target: cell debugContinuation selector: #value; addLine].

	menu 
		add: 'explore value' target: self contents action: #explore; 
		add: 'edit formula' target: self action: #queryFormula;
		addLine.

	"casting"
	menu addLabelledChoice: self castings getter: #casting setter: #casting:.

	menu
		addLine;
	"	add: 'pipe all way through' target: self action: #pipeToInputCells;"
		add: 'keep as single output' target: self action: #keepAsSingleOutput;
		addLine;
		add: 'explore cell' target: self cell action: #explore; 
		addLine; add: 'reset cell' action: #resetCell.

	self isInputCell ifTrue: [

		(MuO context compositionEnvironment hasReactiveTableTemplateForForging: 
				cell ownFormulaValue className)
			ifTrue: [menu addLine; 
						add: 'erase current forge for ', cell ownFormulaValue className 
							target: self action: #eraseReactiveTableTemplateForForging].

			menu
				addLine;
			add: 'store as forge for ', cell ownFormulaValue className 
				target: self action: #storeAsForgingTemplate;
			add: 'hard-code as forge for ', cell ownFormulaValue className 
				target: self action: #hardCodeAsForgingTemplate].

	menu popUpInWorld ! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:20'!
populateWith: anArray

	| fsgen |

	fsgen := anArray generator.
	fsgen next. "skip the grid layout"

	self cellMorphsDo: [:rcm | 
		fsgen next ifNotNil: [:fs | 
			fs key ifNotNil: [
				rcm cell formulaString: fs key evaluatedBy: self table.
				fs value ifNotNil: [rcm casting: fs value].
				rcm updateFromContents]]]! !

!ReactiveCellMorph methodsFor: 'updating' stamp: 'spfa 9/17/2019 18:35'!
precision

	^ 3! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/10/2019 15:07'!
propagateChange

	self dependents do: [:aDependent | 
		(Reactor allFor: aDependent) do: [:ea | ea reactIn: aDependent]]
	! !

!SSSheetBlock methodsFor: 'updating' stamp: 'spfa 9/16/2019 15:11'!
propagateChange

	super propagateChange.
	self dependents do: #propagateChange! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 09:41'!
proxyDictionary
	
	^ ProxyDictionary ifNil: [ProxyDictionary := WeakValueDictionary new]! !

!ReactiveCellMorph methodsFor: 'event handling' stamp: 'spfa 9/15/2019 17:21'!
queryFormula

	| init |

	init := self hasCell not.

	self cell queryFormulaVia: sheet prompt: (Character codePoint: slot x + 64) asString, slot y.

	casting ifNotNil: [self casting: casting].

	(init and: [self hasCell]) ifTrue: [
		"refresh values in output cells, since they may depend on a range of cell, including the one that just appeared"
		self sheet grid relinkOutputCells]! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/15/2019 16:45'!
queryFormulaVia: aPluggableMemory

	self queryFormulaVia: aPluggableMemory prompt: 'Formula' 
! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/15/2019 16:45'!
queryFormulaVia: aPluggableMemory prompt: aString

	| init |

	init := stringFormula.
	init == ReactiveCell notAStringFormula ifTrue: [init := ''].

	self formulaString: 
			((FillInTheBlankMorph reactiveCellRequest: aString initialAnswer: init) 
				ifEmpty: [^ self]) 
		evaluatedBy: aPluggableMemory
! !

!Reactor methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 23:24'!
reactIn: anObject

	[updater valueWithEnoughArguments: {anObject . self source}]
		on: Error
		do: anObject reactorDebugBlock! !

!FillInTheBlankMorph class methodsFor: '*Reactor' stamp: 'spfa 9/15/2019 23:48'!
reactiveCellRequest: queryString initialAnswer: defaultAnswer 

	| aFillInTheBlankMorph |
	aFillInTheBlankMorph
		 := self new setReactiveCellQuery: queryString initialAnswer: defaultAnswer.

	aFillInTheBlankMorph createAcceptButton
		action: [aFillInTheBlankMorph textPane accept].
	aFillInTheBlankMorph createCancelButton
		action: [aFillInTheBlankMorph closeDialog: ''].
		
	[aFillInTheBlankMorph preferredPosition: Sensor cursorPoint] 
		on: Error do: ["fails in Squeak < 5.2"].

	^ aFillInTheBlankMorph getUserResponse
! !

!Exception methodsFor: '*Reactor' stamp: 'spfa 9/15/2019 11:22'!
reactiveTableCopy

	^ self! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/15/2019 11:21'!
reactiveTableCopy

	^ [self deepCopy] on: Error do: [self veryDeepCopy]! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/11/2019 23:22'!
reactorDebugBlock

	^ [:ex | ex signal]! !

!ReactiveCell methodsFor: 'debugging' stamp: 'spfa 9/15/2019 10:11'!
reactorDebugBlock

	^ [:ex | 
		debugContinuation := Continuation current.
		debugContinuation isContinuation
			ifTrue: [	self formulaValue: (ReactorError new updaterError: ex)]
			ifFalse: [ex signal]].
! !

!ReactiveCellMorph methodsFor: 'input' stamp: 'spfa 9/18/2019 01:03'!
recompute

	cell formula: cell formula.
	self updateStringMorph; updateFromContents! !

!ReactiveCell methodsFor: 'conversion' stamp: 'spfa 9/17/2019 15:15'!
reference

	^ self asReactiveReference quoted formulaValue as: AutoQuotingLambdaMessageSend! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/11/2019 10:26'!
referenceProvider

	^ referenceProvider! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/11/2019 10:26'!
referenceProvider: aPluggableMemory

	referenceProvider := aPluggableMemory! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 11:39'!
referencedColumnAt: col

	| cn |

	cn := col.
	cn isInteger ifFalse: [cn := self colNumber: cn].

	^ self columns at: cn ifAbsent: [nil]! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 11:38'!
referencedRowAt: anInteger

	^ self rows at: anInteger ifAbsent: [nil]! !

!ReactiveCell methodsFor: 'initialize-release' stamp: 'spfa 9/11/2019 11:16'!
relink

	self relinkVia: nil! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 11:07'!
relinkCells

	self activeCellsDo: #relink
		! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 15:25'!
relinkOutputCells

	self cellMorphsDo: [:ea | ea isOutputCell ifTrue: [ea cell relink]]
		! !

!ReactiveCell methodsFor: 'initialize-release' stamp: 'spfa 9/11/2019 11:20'!
relinkVia: aPluggableMemory

	stringFormula ifNotNil: [
		aPluggableMemory ifNotNil: [
			^ self formulaString: stringFormula evaluatedBy: aPluggableMemory]].

	self formula: (self formula ifNil: [^ self])! !

!ReactiveCell methodsFor: 'initialize-release' stamp: 'spfa 9/15/2019 13:56'!
reset

	stringFormula := ReactiveCell notAStringFormula.
	formula := formulaValue := debugContinuation := nil.
	self myDependents: #()! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 16:59'!
resetAllOutputBut: aCellMorph

	self cellMorphsDo: [:cm |
		(cm ~~ aCellMorph 
			and: [cm belongsToAReferencedRange not]
			and: [cm isOutputCell]) ifTrue: [cm resetCellWithRelink: false]].

	"refresh values in output cells, since they may depend on a range of cell, including the ones that just disappeared"
	self relinkOutputCells.

	self cellMorphsDo: #updateFromContents.
! !

!ReactiveCellMorph methodsFor: 'event handling' stamp: 'spfa 9/17/2019 14:59'!
resetCell

	self resetCellWithRelink: true! !

!ReactiveCellMorph methodsFor: 'event handling' stamp: 'spfa 9/17/2019 14:58'!
resetCellWithRelink: aBoolean

	casting := nil.
	cell reset.

	self updateStringMorph.
	self formulaMorph contents: ''.

	self sheet grid activeCellsDo: [:c | c removeDependent: cell].

	aBoolean ifTrue: [
		"refresh values in output cells, since they may depend on a range of cell, including the one that just disappeared"
		self sheet grid relinkOutputCells].

	self sheet grid cellMorphsDo: #updateFromContents.

! !

!SSSheetRow methodsFor: 'accessing' stamp: 'spfa 9/10/2019 22:37'!
row: anInteger sheet: aSSSheet

	row := anInteger.
	sheet := aSSSheet.! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 22:40'!
rowAt: anInteger

	^ self rows at: anInteger ifAbsentPut: [SSSheetRow new row: anInteger sheet: self]! !

!ReactiveCellMorph methodsFor: 'querying' stamp: 'spfa 9/12/2019 22:08'!
rowN

	^ slot y! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 22:17'!
rows

	^ rows! !

!FillInTheBlankMorph methodsFor: '*Reactor' stamp: 'spfa 9/11/2019 14:30'!
setReactiveCellQuery: queryString initialAnswer: initialAnswer
	
	| text |
	
	result := initialAnswer ifNil: [''].
	done := false.

	self paneMorph removeAllMorphs.

	self title: queryString.
	
	text := self createTextPaneAcceptOnCR: true.
	self paneMorph addMorphBack: text.

	self paneMorph
		wantsPaneSplitters: false;
		addCornerGrips.
	self paneMorph grips do: [:ea | ea drawCornerResizeHandles: true].
		
	self paneMorph extent: 
		((result asText asMorph extent + (20@10) max: 200@30) min: 500@500).	
	self setDefaultParameters.! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
setter

	^ setter! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
setter: aSelector

	setter := aSelector! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/12/2019 17:57'!
sheet

	^ sheet! !

!SSSheetBlock methodsFor: 'accessing' stamp: 'spfa 9/16/2019 15:42'!
sheet

	^ sheet! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/11/2019 22:24'!
sheet: aReactiveSquares

	sheet := aReactiveSquares.
	self updateFromContents
! !

!SSSheetBlock methodsFor: 'accessing' stamp: 'spfa 9/16/2019 10:09'!
sheet: aSSSheet

	sheet := aSSSheet! !

!ReactiveCellMorph methodsFor: 'querying' stamp: 'spfa 9/12/2019 22:26'!
slot

	^ slot! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/12/2019 22:08'!
slot: aPoint

	slot := aPoint! !

!SSSheet methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 10:24'!
slotNamed: aString

	| col rowi |

	col := self colNumber: aString withoutTrailingDigits.
	rowi := aString asInteger.

	(col isNil or: [rowi isNil]) ifTrue: [^ nil].

	^ col@rowi! !

!Reactor methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 09:17'!
source

	^ source anyOne! !

!Reactor methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 09:17'!
source: anObject updater: uObject

	updater := uObject.
	source:= WeakSet with: anObject! !

!ReactiveCellMorph methodsFor: 'as yet unclassified' stamp: 'spfa 9/28/2019 15:07'!
storeAsForgingTemplate

	MuO context compositionEnvironment
		storeReactiveTableTemplate: self forgeConstructor
		forForging: cell ownFormulaValue className! !

!ReactiveCell methodsFor: 'evaluation' stamp: 'spfa 9/13/2019 09:57'!
stringFormula: aString

	stringFormula := aString! !

!ReactiveCellMorph methodsFor: 'accessing' stamp: 'spfa 9/9/2019 21:50'!
stringMorph

	^ self submorphs first! !

!SSSheetBlock methodsFor: 'querying' stamp: 'spfa 9/11/2019 00:05'!
sum

	^ (self cells collect: #formulaValue) sum! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:16'!
table

	^ table! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/12/2019 00:15'!
table: aReactiveTable

	table := aReactiveTable! !

!ReactiveTable methodsFor: 'as yet unclassified' stamp: 'spfa 2/17/2020 23:35'!
table: aPoint

	^ grid := (ReactiveGrid inARow: (Array streamContents: [:rstr |
		rstr nextPut: ((AlignmentMorph inAColumn: (Array streamContents: [:str1 |
						str1 nextPut: (StringMorph contents: '').
						1 to: aPoint y do: [:n | 
							str1 nextPut: (self cellMorphClass new forRowLabel: n)]])) 
								borderColor: Color transparent; cellInset: 2; layoutInset: 0).
		(1 to: aPoint x) do: [:n |
			| col |
			col := AlignmentMorph inAColumn: (Array streamContents: [:str |
				str nextPut: (StringMorph new contents: (Character codePoint: n + 64) asString).
				(1 to: aPoint y) do: [:m |
					str nextPut: (self cellMorphClass new sheet: self; slot: n@m)]]).
			col borderColor: Color transparent.
			col	layoutInset: 0.
			col cellInset: 2.
			rstr nextPut: col]])) 
				table: self;
				layoutInset: 1;
				cellInset: 0;
				color: Color white! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/10/2019 09:22'!
target

	^ target ifNil: [self]! !

!ReactiveCell methodsFor: 'accessing' stamp: 'spfa 9/10/2019 09:56'!
target: anObject

	target := anObject! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
testAsReactiveReference

	| m ref |

	ref := (m := Morph new) asReactiveReference.

	self assert: ref isLambdaMessageSend.
	self assert: ref default = m.
	self assert: ref parameters anyOne type == ReactiveReference
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 23:45'!
testCircularDependency

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.
 
	cell1 ownFormulaValue: 0.

	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	self should: [cell1 formula: (cell2 reference - 3)] raise: Error! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 11:17'!
testCleanUp

	| s1 s2 s3 |

	s1 := StringMorph new contents: '1'.
	s2 := StringMorph new contents: '0'.
	s3 := StringMorph new contents: '1'.

	Reactor update: s2 after: s1 with: [:n :m | 
		n contents: ((Reactor parentsOf: n) collect: [:s | s contents asInteger]) sum asString].

	Reactor update: s2 after: s3 with: [:n :m | 
		n contents: ((Reactor parentsOf: n) collect: [:s | s contents asInteger]) sum asString].

	self assert: (Reactor all includesKey: s2).

	s2 getProperty: #findMeIfYouCan.

	s1 := s2 := s3 := nil.
	Smalltalk garbageCollectMost.

	self deny: (Reactor all keys anySatisfy: [:k | 
					k isMorph and: [k hasProperty: #findMeIfYouCan]])! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 11:59'!
testCleanUp2

	| cell1 cell2 zo |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
 
	cell1 ownFormulaValue: 0.

	cell2 formula: (cell1 reference + 1).

	cell2 setter: (zo := UUID new).

	self assert: (Reactor all keys anySatisfy: [:k | 
					(k isKindOf: ReactiveCell) and: [k setter == zo]]).

	cell1 := cell2 := nil.
	Smalltalk garbageCollectMost.

	self deny: (Reactor all keys anySatisfy: [:k | 
					(k isKindOf: ReactiveCell) and: [k setter == zo]])! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 11:54'!
testDemo1

	| cell morph |

	morph := Morph new.
	cell := ReactiveCell new formula: (morph asReactiveReference color).

	self assert: cell formulaValue = Color blue.

	morph color: Color yellow.
	
	self assert: cell formulaValue = Color yellow! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
testDemo2

	| cell morph pm |

	morph := Morph new.
	pm := PluggableMemory new.
	pm bindings: ({#mimi -> morph asReactiveReference} as: Dictionary).
	cell := ReactiveCell new formulaString: 'mimi color' evaluatedBy: pm.

	self assert: cell formulaValue = Color blue.

	morph color: Color yellow.
	
	self assert: cell formulaValue = Color yellow! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
testDemo3

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.
 
	cell1 ownFormulaValue: 0.

	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	cell1 formula: 42.
	self assert: cell3 formulaValue= 430.

	cell1 formula: 10.
	self assert: cell3 formulaValue= 110.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 11:17'!
testLowLevel1

	| s1 s2 s3 |

	s1 := StringMorph new contents: '1'.
	s2 := StringMorph new contents: '0'.
	s3 := StringMorph new contents: '1'.

	Reactor update: s2 after: s1 with: [:n :m | 
		n contents: ((Reactor parentsOf: n) collect: [:s | s contents asInteger]) sum asString].

	Reactor update: s2 after: s3 with: [:n :m | 
		n contents: ((Reactor parentsOf: n) collect: [:s | s contents asInteger]) sum asString].

	self assert: s2 contents = '0'.

	s1 contents: '57'.

	self assert: s2 contents = '58'.

	s3 contents: '13'.

	self assert: s2 contents = '70'.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 11:18'!
testLowLevel2

	| a b c |

	a := Morph new.
	b := Morph new.
	c := Morph new.

	Reactor update: b after: a with: [:n :m | n color: m color muchLighter].
	Reactor update: c after: b with: [:n :m | n color: m color muchLighter].

	self assert: a color = Color blue.
	self assert: b color = Color blue.
	self assert: c color = Color blue.

	b color: Color yellow.

	self assert: a color = Color blue.
	self assert: b color = Color yellow.
	self assert: c color = Color yellow muchLighter.

	a color: Color green.

	self assert: a color = Color green.
	self assert: b color = Color green muchLighter.
	self assert: c color = Color green muchLighter muchLighter.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:24'!
testPiping

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.

	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	cell3 formula: (cell3 formula ofVariables: cell3 formula slots) <=!! cell2 formula.

	cell1 formula: 42.
	self assert: cell3 formulaValue= 430.

	cell1 formula: 10.
	self assert: cell3 formulaValue= 110.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/13/2019 23:20'!
testPiping2

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.
 
	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	cell3 formula: (cell3 formula ofVariables: cell3 formula slots) 
						<== (cell3 formula slots collect: [:ea | ea default formula])..

	cell1 formula: 42.
	self assert: cell3 formulaValue= 430.

	cell1 formula: 10.
	self assert: cell3 formulaValue= 110.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/18/2019 00:36'!
testPiping3

	| cell1 cell2 cell3 |

	cell1 := ReactiveCell new.
	cell2 := ReactiveCell new.
	cell3 := ReactiveCell new.

	cell2 formula: (cell1 reference + 1).
	cell3 formula: (cell2 reference * 10).

	cell3 pipeToInputCells.

	self assert: cell3 formula printString = '/o.{(<o> formulaValue + 1) * 10}'.

	cell1 formula: 42.
	self assert: cell3 formulaValue= 430.

	cell1 formula: 10.
	self assert: cell3 formulaValue= 110.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/10/2019 12:11'!
testReactiveReferenceWeakness

	| f g |

	f := Morph new asReactiveReference color.

	Smalltalk garbageCollectMost.
	self should: [f defaultForm value] raise: Error.

	"Contrast this with:"

	g := Morph new asLambdaParameter color.

	Smalltalk garbageCollectMost.
	self assert: g defaultForm value = Color blue.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:39'!
testReactiveTable1

	self assert: ReactiveTable example1 grid valuesForTesting =  
	{5@5 . 1 . 6 . 36 . nil . 12 . nil . nil . nil . nil . nil . '## /x.{<x> sin}' . nil . nil . '## /x.{(<x> + 1) cos}' . '## /x.{(<x> + 1) cos + 12}' . nil . '## /x.{((<x> + 1) cos + 12) sin}' . '## /x.{(<x> sin + 1) cos + 12}' . nil . '12.09124370457176' . 42 . nil . '## /x.{(<x> sin + 1) cos + 12}' . nil . 12.09124370457176}! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:39'!
testReactiveTable2

	| table |

	table := ReactiveTable example1.

	(table grid cellMorphAt: 4@5) keepAsSingleOutput.

	self assert: table grid valuesForTesting = {5@5 . 1 . nil . nil . nil . 12 . nil . nil . nil . nil . nil . '## /x.{<x> sin}' . nil . nil . '## /x.{(<x> + 1) cos}' . nil . nil . nil . nil . nil . '12.09124370457176' . 42 . nil . nil . nil . nil}.

		self assert: table grid constructor = {5@5 . '1'->nil . nil . nil . nil . '12'->nil . nil . nil . nil . nil . nil . 'X  sin'->nil . nil . nil . '(X + 1) cos'->nil . nil . nil . nil . nil . nil . '((C1 o: (C4 + A5)) <~ 0.5)'->#asString . '42'->nil . nil . nil . nil . nil} ! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/18/2019 00:39'!
testReactiveTable3

	| table |

	table := ReactiveTable example2.

	self assert: table grid valuesForTesting =  {5@5 . 1 . 6 . 30 . 5.477225575051661 . nil . nil . 5.477225575051661 . nil . '5.477225575051661' . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}.

	(table grid cellMorphAt: 2@2) keepAsSingleOutput.

	self assert: table grid valuesForTesting =  {5@5 . 1 . nil . nil . nil . nil . nil . 5.477225575051661 . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}.

	self assert: table grid constructor = {5@5 . '1'->nil . nil . nil . nil . nil . nil . '#Unwritten'->#asNumber . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:43'!
testReactiveTable4

	| table |

	table := ReactiveTable example2.

	(table grid cellMorphAt: 2@4) keepAsSingleOutput.

	self assert: table grid valuesForTesting = {5@5 . 1 . nil . nil . nil . nil . nil . nil . nil . '5.477225575051661' . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}.

	self assert: table grid constructor = {5@5 . '1'->nil . nil . nil . nil . nil . nil . nil . nil . '((A1 + 5) * 5) sqrt'->#asString . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil . nil}! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/18/2019 01:06'!
testReactiveTable5

	| table |

	table := ReactiveTable example3.

	(table grid cellMorphAt: 3@5) keepAsSingleOutput.
	self assert: (table grid cellMorphAt: 3@5) stringMorph contents = '#''21'''.
	self assert: (table grid cellMorphAt: 3@5) formulaMorph contents = '#Unwritten'.
	
	(table grid cellMorphAt: 3@5) recompute.
	self assert: (table grid cellMorphAt: 3@5) stringMorph contents = '#''21'''.

	(table grid cellMorphAt: 1@1) formula: 2.
	self assert: (table grid cellMorphAt: 1@1) stringMorph contents = '2'.
	self assert: (table grid cellMorphAt: 1@1) formulaMorph contents = ''.
	self assert: (table grid cellMorphAt: 3@5) stringMorph contents = '#''67'''.

	(table grid cellMorphAt: 1@1) formulaString: '2+7'.
	self assert: (table grid cellMorphAt: 1@1) stringMorph contents = '9'.
	self assert: (table grid cellMorphAt: 1@1) formulaMorph contents = '2+7'.
	self assert: (table grid cellMorphAt: 3@5) stringMorph contents = '#''73'''.
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 11:01'!
testSSSheetColNumber

	| sheet |

	sheet := SSSheet new.

	self assert: (sheet colNumber: 'A') = 1.
	self assert: (sheet colNumber: 'Z') = 26.
	self assert: (sheet colNumber: 'AA') = 27.
	self assert: (sheet colNumber: 'AZ') = 52.
	self assert: (sheet colNumber: 'BA') = 53.
	self assert: (sheet colNumber: 'ZZ') = (27*26)! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 14:53'!
testSpreadSheet0

	| sheet |

	sheet := SSSheet new.

	(sheet \#A1) formula: 12.
	(sheet \#A2) formulaString: 'A1+1'.
	(sheet \#A3) formulaString: 'A2 +1'.
	(sheet \#A4) formulaString: 'A3+1'.

	self assert: (sheet \#A4) formulaValue = 15.

	(sheet \#A1) formula: 10.
	self assert: (sheet \#A4) formulaValue = 13.

! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 14:53'!
testSpreadSheet1

	| sheet |

	sheet := SSSheet new.

	(sheet \#ARY5) formula: 12.
	(sheet \#MU1856) formula: 27.
	(sheet \#Z3) formulaString: 'ARY5 + MU1856'.
	(sheet \#Z4) formulaString: 'ARY5'.

	self assert: (sheet \#Z3) formulaValue = 39.
	self assert: (sheet \#Z4) formulaValue = 12.

	(sheet \#ARY5) formula: 33.
	self assert: (sheet \#Z3) formulaValue = 60.
	self assert: (sheet \#Z4) formulaValue = 33.

	(sheet \#MU1856) formula: -33.
	self assert: (sheet \#Z3) formulaValue = 0.
	! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 14:53'!
testSpreadSheet2

	| sheet |

	sheet := SSSheet new.

	(sheet \#ARY5) formula: 12.
	(sheet \#MU1856) formula: 27.
	(sheet \#Z3) formulaString: 'ARY5 + MU1856'.

	(sheet \#Z4) formulaString: 'colMU sum'.

	(sheet \#MU1856) formula: 207.

	self assert: (sheet \#Z4) formulaValue = 207.

	(sheet \#MU12) formula: -7.

	self assert: (sheet \#Z4) formulaValue = 200.

	(sheet \#Z5) formulaString: 'colMU loc: 1856'.
	self assert: (sheet \#Z5) formulaValue = 207.

	(sheet \#Z5) formulaString: 'row1856 loc: 359'.
	self assert: (sheet \#Z5) formulaValue = 207.! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 14:53'!
testSpreadSheet3

	| sheet |

	sheet := SSSheet new.

	(sheet \#A5) formula: 12.
	(sheet \#B6) formula: 10.
	(sheet \#C7) formula: 20.

	(sheet \#Z4) formulaString: 'A5toC7 sum'.
	self assert: (sheet \#Z4) formulaValue = 42.

	(sheet \#A5) formula: 2.
	self assert: (sheet \#Z4) formulaValue = 32.

	(sheet \#Z5) formulaString: 'A5toC7 loc: 3@3'.
	self assert: (sheet \#Z5) formulaValue = 20.

	self assert: sheet matrices size = 1.

	(sheet \#C7) formula: -10.
	self assert: (sheet \#Z5) formulaValue = -10.

	(sheet \#B6) formula: -10.
	self assert: (sheet \#Z4) formulaValue = -18.

! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:25'!
testSpreadSheet4

	| sheet |

	sheet := SSSheet new.

	(sheet \#A5) formula: 12.
	(sheet \#B6) formula: 10.
	(sheet \#C7) formula: 20.

	(sheet \#Z4) formulaString: 'A5toC7'.
	self assert: ((sheet \#Z4) formulaValue isKindOf: SSSheetMatrix).

	(sheet \#Z3) formulaString: 'A5toC7 sum'.
	self assert: (sheet \#Z3) formulaValue = 42.

	(sheet \#Z5) formulaString: 'Z4 sum'.
	self assert: (sheet \#Z5) formulaValue = 42.

	(sheet \#Z4) formulaString: 'A5toB6'.
	self assert: (sheet \#Z5) formulaValue = 22.

	(sheet \#B6) formula: -10.
	self assert: (sheet \#Z3) formulaValue = 22.

	self assert: (sheet \#Z5) formulaValue = 2.
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 14:25'!
testSpreadSheet4b

	| sheet  |

	sheet := SSSheet new.

	(sheet \#A5) formula: 12.
	(sheet \#B6) formula: 10.
	(sheet \#C7) formula: 20.

	(sheet \#Z4) formulaString: 'A5toC7 sum'.

	(sheet \#Z3) formulaString: 'A5toC7 sum'.
	self assert: (sheet \#Z3) formulaValue = 42.

	(sheet \#Z5) formulaString: 'Z4'.
	self assert: (sheet \#Z5) formulaValue = 42.

	(sheet \#Z4) formulaString: 'A5toB6 sum'.
	self assert: (sheet \#Z5) formulaValue = 22.

	(sheet \#B6) formula: -10.
	self assert: (sheet \#Z3) formulaValue = 22.
	self assert: (sheet \#Z5) formulaValue = 2.
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:27'!
testSpreadSheet4c

	| sheet |

	sheet := SSSheet new.

	(sheet \#A5) formula: 12.
	(sheet \#B6) formula: 10.
	(sheet \#C7) formula: 20.

	(sheet \#Z4) formulaString: 'A5toC7'.
	self assert: ((sheet \#Z4) formulaValue isKindOf: SSSheetMatrix).

	(sheet \#Z3) formulaString: 'A5toC7 sum'.
	self assert: (sheet \#Z3) formulaValue = 42.

	(sheet \#Z5) formulaString: 'Z4 sum'.
	self assert: (sheet \#Z5) formulaValue = 42.

	(sheet \#Z6) formulaString: 'Z4 values'.
	self assert: (sheet \#Z6) formulaValue = #(20 10 12).

	(sheet \#Z4) formulaString: 'A5toB6'.
	self assert: (sheet \#Z5) formulaValue = 22.

	(sheet \#B6) formula: -10.
	self assert: (sheet \#Z3) formulaValue = 22.
	self assert: (sheet \#Z6) formulaValue = #(-10 12).
	self assert: (sheet \#Z5) formulaValue = 2.
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 15:46'!
testSpreadSheet5

	"Meta-stuff. Don't do it."

	| sheet |

	sheet := SSSheet new.

	(sheet \#A5) formula: 12.
	(sheet \#B6) formula: 10.
	(sheet \#C7) formula: 20.

	(sheet \#Z4) formulaString: 'sheet'.
	self assert: (sheet \#Z4) formulaValue sheet == sheet.

	(sheet \#Z4) formulaString: 'sheet cells quoted size '.
	self assert: (sheet \#Z4) formulaValue = 4.
! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 9/16/2019 16:17'!
testSpreadSheet6

	| sheet |

	sheet := SSSheet new.

	1 to: 100 do: [:n |
		1 to: 100 do: [:m |
			sheet cellAt: n@m put: n+m]].

	self assert: (sheet \#AX85) formulaValue = 135.

	self assert: (sheet cellAt: 25@17) formulaValue = 42.

	(sheet cellAt: 101@4) formulaString: 'A1toZ85 sum'.
	self assert: (sheet cellAt: 101@4) formulaValue = 124865.

	(sheet cellAt: 101@6) formulaString: 'A1toZ85 values average'.
	self assert: (sheet cellAt: 101@6) formulaValue =  (113/2).

	(sheet cellAt: 101@2) formulaString: 'colA sum'.
	self assert: (sheet cellAt: 101@2) formulaValue = 5150.

	(sheet cellAt: 101@3) formulaString: 'row1 sum'.
	self assert: (sheet cellAt: 101@3) formulaValue = 5150

! !

!ReactorTest methodsFor: 'as yet unclassified' stamp: 'spfa 7/25/2022 22:14'!
testWeakLambdaSlot

	| slot |

	slot := (MagnifierMorph new 
					setProperty: #weirdStuff toValue: 42;
					yourself) 
				asReactiveReference parameters anyOne slot.

	self assert: slot class == WeakLambdaSlot.

	self deny: (MagnifierMorph allInstances noneSatisfy: [:cm | cm hasProperty: #weirdStuff]).

	Smalltalk garbageCollectMost.

	self assert: (MagnifierMorph allInstances noneSatisfy: [:cm | cm hasProperty: #weirdStuff]).! !

!SSSheetMatrix methodsFor: 'accessing' stamp: 'spfa 9/16/2019 09:52'!
topLeft: aPoint

	topLeft := aPoint! !

!Reactor class methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 10:07'!
update: anObject after: sObject with: aBlock

	(anObject == sObject 
		or: [	anObject deepDependents includes: sObject]) 
			ifTrue: [self error: 'Circular dependency.'].

	sObject addDependent: anObject.

	(Parents at: anObject ifAbsentPut: [WeakSet new]) 
		add: sObject.

	(Reactors at: anObject ifAbsentPut: [IdentitySet new]) 
		add: (Reactor new source: sObject updater: aBlock)! !

!ReactiveCellMorph methodsFor: 'updating' stamp: 'spfa 9/15/2019 16:49'!
updateFromContents

	self eventHandler: nil.
	self formulaMorph eventHandler: nil.
	self color: sheet cellColor; borderColor: sheet cellColor darker.
	self on: #mouseUp send: #yourself to: self.

	(self hasCell ifFalse: [self] ifTrue: [self formulaMorph]) 
		on: #mouseUp send: #editFormulaEvt: to: self.

	self hasCell ifFalse: [^ self].

	self on: #mouseUp send: #popUpMenuEvt: to: self.

	self isInputCell ifTrue: [^ self borderColor: Color orange].
	self isOutputCell ifFalse: [^ self].
	self isIntermediateCell ifFalse: [^ self borderColor: Color blue].
	self borderColor: sheet cellColor blacker! !

!ReactiveCellMorph methodsFor: 'updating' stamp: 'spfa 9/14/2019 23:27'!
updateStringMorph

	self cell formulaString = ReactiveCell notAStringFormula ifTrue: ["signals the cell is reset"
		^ self stringMorph contents: ''].

	self contents isNumber ifFalse: [
		^ self stringMorph contents: (self contents printStringLimitedTo: self maxStringLength)].

	self stringMorph contents: 
		(self contents precision: self precision) asIntegerWhenClose asString.
! !

!Reactor methodsFor: 'as yet unclassified' stamp: 'spfa 9/9/2019 21:50'!
updater

	^ updater! !

!ReactorError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:10'!
updaterError

	^ updaterError! !

!ReactorError methodsFor: 'as yet unclassified' stamp: 'spfa 9/15/2019 10:09'!
updaterError: anError

	updaterError := anError! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/9/2019 21:50'!
updatingLambda: aLMS

	self updatingLambda: aLMS trigger: false! !

!Object methodsFor: '*Reactor' stamp: 'spfa 9/10/2019 11:55'!
updatingLambda: aLMS trigger: aBoolean

	"Give the receiver an unique dependency on all parametric slot default values in a LMS.
Each time one of these object is #changed (or sends #propagateChange) the LMS will be evaulated in its defaultForm with argument the receiver
This means the LMS must be a function of one single argument
If aLMS is nil, the receiver will not have any registered updating action in Reactor"

	| deff parents |

	Reactor free: self.
	aLMS ifNil: [^ self].

	deff := aLMS weakDefaultForm compiled.

	(parents := aLMS parameters collect: #default) do: [:p |
		Reactor update: self after: p with: deff].

	aBoolean ifFalse: [^ self].

	"instant updating"
	(parents ifEmpty: [^ deff value: self]) anyOne propagateChange
	
	
	
! !

!SSSheetBlock methodsFor: 'querying' stamp: 'spfa 9/16/2019 15:29'!
values

	"Note: the returned values are not sorted"

	^ self cells collect: #formulaValue! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/17/2019 18:11'!
valuesForTesting

	^ Array streamContents: [:str | 
		str nextPut: self cellsLayout.
		self cellMorphsDo: [:rcm | 
			str nextPut: (rcm hasCell ifFalse: [nil] 
				ifTrue: [	rcm contentsForTesting])]]! !

!ReactiveGrid methodsFor: 'as yet unclassified' stamp: 'spfa 9/11/2019 11:02'!
veryDeepCopy

	^ super veryDeepCopy relinkCells! !
Reactor initialize!
