SystemOrganization addCategory: #'Sound-Scores-MIDIFileWriter'!


Object subclass: #MIDIFileWriter
	instanceVariableNames: 'stream score trackData activeEvents lastEventTime fileData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sound-Scores-MIDIFileWriter'!

MIDIFileWriter class
	instanceVariableNames: ''!


!MIDIFileWriter commentStamp: 'YYL 3/29/2001 03:18' prior: 0!
MIDIFileWriter 1.01
converts a MIDIScore into a midifile...YAY!!
To load in, modify, and save out a midi file, do this:

(scorePlayerMorph _ ScorePlayerMorph onMIDIFileNamed:'inputMidiFile.mid') openInWorld.
"play with the midi file...move notes around, etc."
"when done, do this:"
mfw _ MIDIFileWriter saveFileNamed: 'outputMidiFile.mid' fromScore: ((scorePlayerMorph scorePlayer) score).
mfw writeMidi.


To simply test out how squeak converts your midifile, try this out:

midiScore _ MIDIFileReader scoreFromFileNamed: 'inputMidiFile.mid'
mfw _ MIDIFileWriter saveFileNamed: 'outputMidiFile.mid' fromScore: midiScore.
mfw writeMidi.

questions? comments? bugs? feature requests? code updates? check out the Knowledge Base
http://guzdial.cc.gatech.edu:8080/squeakers.179
gte356h@prism.gatech.edu!
]style[(797)!

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/21/2001 05:42'!
get24Bit: num
	"returns an array of num as a 24-bit number (with leading 0s)"

	| return |
	return := Array new: 3.
	return at:1 put: (num digitAt: 3).
	return at:2 put: (num digitAt: 2).
	return at:3 put: (num digitAt: 1).
	^return.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/23/2001 22:13'!
noteOffEvents: time
	"returns an ordered collection with the midi file info for all the notes that should be turned off before time"

	| eventData |
	eventData := OrderedCollection new.
	(activeEvents isEmpty) ifFalse:
	[
		[(activeEvents isEmpty)not and: [time >= ((activeEvents first) endTime)]] whileTrue:
			[ |first|
			first := activeEvents removeFirst.
			self assert: [(first channel) <= 16r0F].
			self assert: [(first midiKey) <= 16r7F].
			"add varLengthInt of note off's delta-time to eventData"
			eventData addAllLast: (self varLengthInt: ((first endTime) - lastEventTime)).
			eventData add: (16r80 bitOr: (first channel)).
			eventData add: (first midiKey).
			eventData add: 16r40. "default release velocity"
			lastEventTime := first endTime.
			].
		].
	^eventData.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/23/2001 22:13'!
remainingNoteOffEvents
	"returns an ordered collection with the midi file info for all the remaining notes that should be turned off"

	(activeEvents isEmpty)
	ifFalse:
		[^ self noteOffEvents: ((activeEvents last) endTime).]
	ifTrue:
		[^ OrderedCollection new. "return an empty OC"].! !

!MIDIFileWriter class methodsFor: 'as yet unclassified' stamp: 'YYL 1/15/2001 22:45'!
saveFileNamed: fileName fromScore: aScore

	| t |
	t := self new.
	t stream: (StandardFileStream newFileNamed: fileName) binary.
	t score: aScore.
	^ t.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/11/2001 20:42'!
score: aScore

	score := aScore.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/11/2001 20:40'!
stream: aStream

	stream := aStream.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/21/2001 05:08'!
type
	"returns the midi file type (0,1,or 2)...right now only 0 and 1 are figured out."

	(((score tracks) size = 1) and: [(score tempoMap) isEmptyOrNil])
		ifTrue: [^0.]
		ifFalse: [^1].! !

!MIDIFileWriter class methodsFor: 'as yet unclassified' stamp: 'YYL 1/19/2001 12:37'!
withScore: aScore
"non-functional...for testing only!!"
	| t |
	t := self new.
	"t stream: (StandardFileStream newFileNamed: 'c:\temp\output.mid') binary."
	t score: aScore.
	^ t.! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/15/2001 22:57'!
write16Bit: num
	"writes num as a 16-bit number (with leading 0s"

	stream nextPut: (num digitAt: 2).
	stream nextPut: (num digitAt: 1).! !

!MIDIFileWriter methodsFor: 'as yet unclassified' stamp: 'YYL 1/15/2001 22:57'!
write32Bit: num
	"writes num as a 32-bit number (with leading 0s"

	stream nextPut: (num digitAt: 4).
	stream nextPut: (num digitAt: 3).
	stream nextPut: (num digitAt: 2).
	stream nextPut: (num digitAt: 1).! !

!MIDIFileWriter methodsFor: 'chunk writing' stamp: 'YYL 1/21/2001 05:52'!
writeHeaderChunk
	"writes the midi file's header"

	|type|
	type := self type.
	stream nextPutAll: 'MThd'.
	self write32Bit: 16r06.
	self write16Bit: type.
	(type = 0)
		ifTrue: [self write16Bit: 1.].
	(type = 1)
		ifTrue: [self write16Bit: ((score tracks) size + 1"for TempoMap").].
	self write16Bit: (score ticksPerQuarterNote). "does not support SMPTE"
	! !

!MIDIFileWriter methodsFor: 'chunk writing' stamp: 'YYL 1/23/2001 20:53'!
writeMidi
	"writes the midi file"

	self writeHeaderChunk.
	(self type) = 1 ifTrue:
		[self writeTrackChunk: (score tempoMap) named: 'TempoMap'].
	1 to: ((score tracks) size) do:
	[:trackNo| self writeTrackChunk: ((score tracks) at: trackNo) named: ((score trackInfo) at: trackNo).
	].
	stream close.! !

!MIDIFileWriter methodsFor: 'chunk writing' stamp: 'YYL 3/29/2001 01:46'!
writeTrackChunk: track named: trackName
	| sortedTrack |
	"writes the midi file's Track info"

	trackData := OrderedCollection new.

	stream nextPutAll: 'MTrk'.
	activeEvents := SortedCollection new.
	activeEvents sortBlock: [:n1 :n2 | (n1 endTime) <= (n2 endTime)].
	lastEventTime := 0.

"FIX: March 29,2001: sort track before do each event...this fixes problems with notes being out of order in the track.  Out of order notes lead to writing negative time values in the midi file..a definite no no!!  Added the following line to sort it."
	sortedTrack := (track asSortedCollection:[:n1 :n2 | (n1 time) <= (n2 time)]) asArray.

	sortedTrack do:"build track info in trackData here"
		[:event|
		trackData addAllLast: (self convertEvent: event).
		].
	trackData addAllLast: (self remainingNoteOffEvents).
	(activeEvents isEmpty) ifFalse: [self error:'still activeEvents left!!'.].
	trackData add: 0. trackData add: 255. trackData add:47. trackData add: 0. "end of track meta-event"
	self write32Bit: (trackData size).
	trackData do:
		[:current |
		stream nextPut: current.
		].
	! !
